'
' Minimalist FAT16 read routines
' The code uses no large sector buffers
' Written by Les Johnson. September 2011
'
    Declare Reminders = Off
    Declare Warnings = Off
    Declare Unsigned_Dwords = On                            ' Turn off the extra code for signed Dword variables

    Dim Sector_bSkipLoop As Byte

    Dim BPB_dStartSector As Dword
    Dim BPB_wResSectorCount As Word
    Dim BPB_wFAT_Start As Word                              ' Starting Sector of the FAT
    Dim BPB_bNumberOf_FATs As Byte                          ' Number of FATs
    Dim BPB_bSectorsPerCluster As Byte                      ' Number of sectors per cluster
    Dim BPB_wFAT_Size As Word                               ' Size of FAT
    Dim BPB_wRootDirEntryCount As Word

    Dim FAT_ByteOut As BPB_bNumberOf_FATs
    Dim FAT_wStartSector As Word
    Dim FAT_wFirstDataSector As Word                        ' Data starting sector
    Dim FAT_wRootDirStartSector As Word                     ' First sector of root directory
    Dim FAT_wRootDirNumSectors As Word                      ' Number of sectors allocated to root directory

    Dim Dir_dCurrentSector As Dword
    Dim Dir_bCurrentSectorInCluster As Byte
    Dim Dir_bFileNumber As Word
    Dim Dir_bResponse As Byte
'
' Current File state
'
    Dim FAT_bFileAttr As Byte
    Dim FAT_dFileSize As Dword
    Dim FAT_bCurrentSectorInCluster As Byte
    Dim File_dCurrentByteCount As Dword System
    Dim FAT_wFileNumber As Word
    Dim FAT_WordIn As Word System
    Dim FAT_ByteIn As FAT_WordIn.Byte0
    Dim FAT_dFreeSpace As Dword

    Dim File_wStartCluster As Word
    Dim File_wCurrentCluster As Word

    Dim bListResponse As Byte

    Dim FAT_bResponse As Byte
    Dim tStartedListingDirectory As Bit
    Dim tStartedReadingFile As Bit

    Dim PP0 As Byte System
    Dim PP0H As Byte System
    Dim FAT_wEntry As PP0.Word

    Dim List_wInnerLoop As FAT_wFileNumber                  ' Alias to variable FAT_wFileNumber
    Dim List_wOuterLoop As Dir_bFileNumber                  ' Alias to variable Dir_bFileNumber
    Dim Sector_wReadLoop As Word System

    Dim Sector_wByteCounter As Word System                  ' Counts the bytes read from the sector
    Dim Sector_wFinishLoop As Word System                   ' Forms a loop to read the remaining bytes from the sector

    Dim FAT_Filename As String * 12 Heap
    Dim FAT_sInternalFilename As String * 12 Heap

    Dim FAT_wFSR0 As FSR0L.Word
    Dim FAT_wFSR1 As FSR1L.Word
    Dim FAT_wPROD As PRODL.Word
'
' Error response values
'
    Symbol cFAT_NoMoreFiles      = $00
    Symbol cFAT_ValidFile        = $01
    Symbol cFAT_EndofOfDirectory = $02
    Symbol cFAT_LongFilename     = $03
    Symbol cFAT_EndOfFile        = $00
    Symbol cFAT_FileNotFound     = $0B
    Symbol cFAT_IsDirectory      = $10
    Symbol cFAT_FileDeleted      = $E5

$ifndef True
    $define True 1
$endif
$ifndef False
    $define False 0
$endif

'--------------------------------------------------------------------------------------------

    Include "SD.inc"                                    ' Load the SD card routines

'--------------------------------------------------------------------------------------------
    GoTo _FAT_Main_                                     ' Jump over the subroutines
'----------------------------------------------------------------------------------------------------------
' Skip x amount of bytes within the current sector
' Input     : WREG holds the amount of skips (dummy reads) to perform
' Output    : SD_SPI_Read(tWithIncrement) macro counts the bytes skipped and places in variable Sector_wByteCounter
' Notes     : None
'
$define Sector_Skip_Bytes(pAmount) '
    WREG = pAmount                 '
    GoSub _Sector_Skip_Bytes

_Sector_Skip_Bytes:
    Sector_bSkipLoop = WREG
    If WREG = 0 Then Return
    Repeat
        Dec Sector_bSkipLoop
        SD_SPI_Read(tWithIncrement)
    Until Sector_bSkipLoop = 0
    Return

'----------------------------------------------------------------------------------------------------------
' After enabling the card, use this routine to extract the drive info and find the FAT and Root directory
' Input     : None
' Output    : Returns with WREG holding 0 if the card cannot be read
' Notes     : In order to save code space, it assumes that there are 512 bytes per sector
'
$define FAT_Init() _FAT_Init

_FAT_Init Macro- \Byte
    GoSub __FAT_Init
    Return_Wreg
    Endm

__FAT_Init:
    tStartedListingDirectory = False
    SD_Init()                                                   ' Initialise the SD card
    'tFAT_Response = True                                       ' Default to everything OK
    If tSD_Response = False Then
        WREG = 0
        Return
    EndIf
'
' Get the Master Boot Record
'
    Sector_wByteCounter = 0
    SD_PointToSector(0)
'
' Move to the partition type with the sector (Offset 450)
'
    Sector_Skip_Bytes(200)
    Sector_Skip_Bytes(250)
'
' Check if the partition type is acceptable. (must be FAT16)
'
    Select  SD_SPI_Read(tWithIncrement)
        Case $04, $06, $0E              ' Is the partition FAT16?
            WREG = 1                    ' Yes. So accept it
        Case Else                       ' Is the partition FAT12 or FAR32 or unknown?
            WREG = 0                    ' Yes. So signal an error
            Return                      ' Exit the routine
    EndSelect
'
' Go to the partition offset value within the sector (Offset 454)
'
    Sector_Skip_Bytes(3)
'
' Read the 32-bit sector value pointing to the partition sector
'
    BPB_dStartSector.Byte0 = SD_SPI_Read(tWithIncrement)
    BPB_dStartSector.Byte1 = SD_SPI_Read(tWithIncrement)
    BPB_dStartSector.Byte2 = SD_SPI_Read(tWithIncrement)
    BPB_dStartSector.Byte3 = SD_SPI_Read(tWithIncrement)
'
' Move to the partition sector
'
    SD_PointToSector(BPB_dStartSector)

    Sector_Skip_Bytes(13)                                              ' Go to the Sectors per cluster offset within the sector (Offset 13)

    BPB_bSectorsPerCluster = SD_SPI_Read(tWithIncrement)               ' Get the size of a cluster (8-bit)

    BPB_wResSectorCount.Byte0 = SD_SPI_Read(tWithIncrement)
    BPB_wResSectorCount.Byte1 = SD_SPI_Read(tWithIncrement)

    BPB_bNumberOf_FATs = SD_SPI_Read(tWithIncrement)                   ' Get the number of FATs (8-bit)

    BPB_wRootDirEntryCount.Byte0 = SD_SPI_Read(tWithIncrement)         ' \ Get the maximum number of entries in the root directory
    BPB_wRootDirEntryCount.Byte1 = SD_SPI_Read(tWithIncrement)         ' /

    Sector_Skip_Bytes(3)                                               ' Go to the FAT size offset within the sector (Offset 22)
    BPB_wFAT_Size.Byte0 = SD_SPI_Read(tWithIncrement)                  ' \ Determine the number of sectors in one FAT
    BPB_wFAT_Size.Byte1 = SD_SPI_Read(tWithIncrement)                  ' /
'
' Uncomment for a diagnostic of the FAT format
'
(*
    HRSOut "Start Sector = ", Dec BPB_dStartSector,13
    HRSOut "Sectors Per Cluster = ", Dec BPB_bSectorsPerCluster,13
    HRSOut "BPB_wResSectorCount = ", Dec BPB_wResSectorCount, 13
    HRSOut "Number Of FATS = ", Dec BPB_bNumberOf_FATs,13
    HRSOut "Root Dir Entry Count = ", Dec BPB_wRootDirEntryCount,13
    HRSOut "Fat Size = ", Dec BPB_wFAT_Size,13
*)
'
' Calculate the location of the FAT and ROOT directory
'
    FAT_wStartSector = BPB_dStartSector + BPB_wResSectorCount
    FAT_wRootDirNumSectors = BPB_wRootDirEntryCount * 32
    FAT_wRootDirNumSectors = FAT_wRootDirNumSectors + 511            ' \
    FAT_wRootDirNumSectors = FAT_wRootDirNumSectors / 512            ' / Assumes 512 bytes per sector
    FAT_wRootDirStartSector = BPB_bNumberOf_FATs * BPB_wFAT_Size
    FAT_wRootDirStartSector = FAT_wRootDirStartSector + FAT_wStartSector
    FAT_wFirstDataSector = FAT_wRootDirStartSector + FAT_wRootDirNumSectors
    WREG = 1
    Return
'----------------------------------------------------------------------------------------------------------
' Open a file for reading
' Input     : String FAT_Filename holds the file to open
' Output    : Returns with WREG holding cFAT_FileNotFound if the file cannot be found
' Notes     : Seperates the extension from the filename by looking for the "."
'           : Then pads the filename with spaces if required
'
$define File_Open(pFilename) _OpenFile pFilename

_OpenFile Macro pFilename
    #if((Prm_1 == Char) || (Prm_1 == Label))
        Movlw   (pFilename >> 8)
        Movwf   TBLPTRH
        Movlw   (pFilename & 255)
        Movwf   TBLPTRL
        Lfsr    0,FAT_Filename
        Clrf    WREG
        Tblrd*+
        Movf    TABLAT,W
        Movwf   POSTINC0
        Bz      $ + 6
        Decfsz  WREG,F
        Bra     $ - 10
        Num_Word FAT_Filename,FAT_wEntry
    #endif
    #if((Prm_1 == Word) || (Prm_1 == Dword))
        Word_Word pFilename,TBLPTRL
        Lfsr    0,FAT_Filename
        Clrf    WREG
        Tblrd*+
        Movf    TABLAT,W
        Movwf   POSTINC0
        Bz      $ + 6
        Decfsz  WREG,F
        Bra     $ - 10
        Num_Word FAT_Filename,FAT_wEntry
    #endif
    #if((Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32) || (Prm_1 == String))
        Num_Word pFilename,FAT_wEntry
    #endif
    GoSub __FAT_OpenFile
    Return_Wreg
    Endm

__FAT_OpenFile:
'
' Remove the "." and pad the filename with spaces
'
    FAT_wFSR0 = FAT_wEntry                                      ' Point FSR0 to the start of FAT_wEntry
    FAT_wFSR1 = VarPtr FAT_sInternalFilename                    ' Point FSR1 to the start of FAT_sInternalFilename
    PRODL = 0                                                   ' Clear the loop counter
    PRODH = 0
    Repeat                                                      ' Create a loop to scan FAT_Filename
        Inc PRODL                                               ' Increment the loop counter
        POSTINC1 = POSTINC0                                     ' Copy the filename into FAT_sInternalFilename
        If INDF0 = "." Then Break                               ' Exit the loop when . found
        Inc PRODH
    Until PRODH >= 11

    Repeat                                                      ' Create a loop to pad out with spaces
        POSTINC1 = " "                                          ' Pad it
        Inc PRODH
    Until PRODH >= 7
    WREG = POSTINC0                                             ' Discard the "."
    POSTINC1 = POSTINC0                                         ' \
    POSTINC1 = POSTINC0                                         ' | Copy extension into FAT_sInternalFilename
    POSTINC1 = POSTINC0                                         ' /
    INDF1 = 0                                                   ' Add a null to FAT_sInternalFilename
    FAT_Filename = ToUpper FAT_sInternalFilename                ' Convert the filename to Uppercase
'
' Open a file without padding or altering it's name
' The file's name should already be in string FAT_Filename
'
FAT_OpenFile_Raw:
'
' Look in the Current Working Directory for a file with a given name
' Upon exiting the loop, FAT_wFileNumber will hold the number of the file
'
    FAT_wFileNumber = 0
    While 1 = 1                                                 ' Create a loop to scan the files
        Dir_bFileNumber = FAT_wFileNumber
        GoSub FAT_Load_File_in_CWD                              ' Attempt to load a file from the Current Working Directory with slot number Dir_bFileNumber
        If Dir_bResponse = cFAT_EndofOfDirectory Then           ' Have we reached the end of the directory?
            WREG = cFAT_FileNotFound                            ' Yes. So issue an error value
            Return                                              ' Exit the subroutine if file not found
        EndIf
        If Dir_bResponse <> cFAT_FileDeleted Then
            If Dir_bResponse <> cFAT_LongFilename Then
                If FAT_sInternalFilename = FAT_Filename Then    ' Have we found the file?
                    FAT_bResponse = 1                           ' Yes. So indicate it
                    Break                                       ' Exit the loop
                EndIf
            EndIf
        EndIf
        Inc FAT_wFileNumber
    Wend
    File_dCurrentByteCount = 0
    FAT_bCurrentSectorInCluster = 0
    File_wCurrentCluster = File_wStartCluster
'
' Find the working sector within the current cluster
'
FAT_ConvertSectorToCluster:
    SD_dCurrentSector = File_wCurrentCluster - 2                            ' First two FAT entries are not data!
    SD_dCurrentSector = SD_dCurrentSector * BPB_bSectorsPerCluster          ' Find the sector num relative to the data start
    SD_dCurrentSector = SD_dCurrentSector + FAT_wFirstDataSector            ' Find absolute sector number
    SD_dCurrentSector = SD_dCurrentSector + FAT_bCurrentSectorInCluster     ' Add the offset sector within the cluster
    SD_PointToSector(SD_dCurrentSector)                                     ' Point to the sector within the SD. Exit via the subroutine
    tStartedReadingFile = False
    WREG = FAT_bResponse
    Return
'----------------------------------------------------------------------------------------------------------
' Return with the files in the current directory
' Input      : None
' Output     : String FAT_sInternalFilename holds the name of the file
'            : FAT_dFileSize holds the size of the file (in bytes)
'            : FAT_dFreeSpace holds the remaining bytes on the disk (in kB)
'            : bListResponse holds the state of the directory
'            : 255 for an error
'            : 0 for End Of Directory
'            : 1 for Valid File
' Notes      : None
'
$define Dir_List() _FAT_ListDir

_FAT_ListDir Macro- \Byte
    GoSub __FAT_ListDir
    Return_Wreg
Endm

__FAT_ListDir:
'
' Find the amount of free clusters (i.e. those with 0 as the value in FAT)
'
    bListResponse = 255
    If tStartedListingDirectory = False Then
        FAT_dFreeSpace = 0
        List_wOuterLoop = 0
        Repeat
            SD_PointToSector(FAT_wStartSector + List_wOuterLoop)
            For List_wInnerLoop = 255 To 0 Step -1
                FAT_wEntry.Byte0 = SD_SPI_Read(tWithIncrement)
                FAT_wEntry.Byte1 = SD_SPI_Read(tWithIncrement)
                If FAT_wEntry = 0 Then Inc FAT_dFreeSpace
            Next
            Inc List_wOuterLoop
        Until List_wOuterLoop > BPB_wFAT_Size
        FAT_dFreeSpace = FAT_dFreeSpace * 2
        tStartedListingDirectory = True
        FAT_wFileNumber = 0
    Else
        Dir_bFileNumber = FAT_wFileNumber
        GoSub FAT_Load_File_in_CWD
        If Dir_bResponse = cFAT_EndofOfDirectory Then       ' Have we reached the end of the directory ?
            tStartedListingDirectory = False                ' Yes. So reset the directory listing mechanism
            bListResponse = 0
            WREG = cFAT_NoMoreFiles
            Return
        EndIf
        If Dir_bResponse <> cFAT_FileDeleted Then
            If Dir_bResponse <> cFAT_LongFilename Then      ' Is it a potential valid file name?
                If FAT_bFileAttr.4 = 0 Then                 ' Is it a directory ?
                    bListResponse = 1                       ' No. So it's a standard file name
                EndIf
            EndIf
        EndIf
        Inc FAT_wFileNumber                                 ' Move to the next file in the directory
    EndIf
    WREG = bListResponse
    Return

'----------------------------------------------------------------------------------------------------------
' Attempt to load a file from the Current Working Directory with slot number Dir_bFileNumber
' Returns Dir_bResponse
'
FAT_Load_File_in_CWD:
    Dir_dCurrentSector = FAT_wRootDirStartSector
    Dir_bCurrentSectorInCluster = 0
    If Dir_bFileNumber > 16 Then
        While Dir_bFileNumber > 16
            Inc Dir_dCurrentSector                          ' Root is a special case, all the sectors are in a row.
            Dir_bFileNumber = Dir_bFileNumber - 16
        Wend
    EndIf
'
' Start reading the directory sector
'
    SD_PointToSector(Dir_dCurrentSector)
    Sector_Skip_Bytes(Dir_bFileNumber << 4)                 ' \
    Sector_Skip_Bytes(Dir_bFileNumber << 4)                 ' / Perform a 16-bit skip

    FAT_ByteIn = SD_SPI_Read(tWithIncrement)
    If FAT_ByteIn = 0 Then                                  ' End of directory ?
        Dir_bResponse = cFAT_EndofOfDirectory               ' Yes. So indicate it
        Return                                              ' Exit the subroutine prematurely
    EndIf
    If FAT_ByteIn = cFAT_FileDeleted Then                   ' Deleted file ?
        Dir_bResponse = cFAT_FileDeleted                    ' Yes. So indicate it
        Return                                              ' Exit the subroutine prematurely
    EndIf
'
' Read file information from the directory
'
    FAT_wFSR0 = VarPtr FAT_sInternalFilename                ' Point FSR0L\H to the FAT_sInternalFilename String
    POSTINC0 = FAT_ByteIn
    FAT_ByteIn = 0
    Repeat
        POSTINC0 = SD_SPI_Read(tWithIncrement)
        Inc FAT_ByteIn
    Until FAT_ByteIn > 9
    POSTINC0 = 0                                            ' Add a Null
    FAT_sInternalFilename = ToUpper FAT_sInternalFilename   ' Convert the file name to Uppercase

    FAT_bFileAttr =  SD_SPI_Read(tWithIncrement)
    If FAT_bFileAttr = $0F Then                             ' Long filename ?
        Dir_bResponse = cFAT_LongFilename                   ' Yes. So indicate it
        Return                                              ' Exit the subroutine prematurely
    EndIf
    Sector_Skip_Bytes(14)
    File_wStartCluster.Byte0 = SD_SPI_Read(tWithIncrement)
    File_wStartCluster.Byte1 = SD_SPI_Read(tWithIncrement)
    File_wCurrentCluster = File_wStartCluster

    FAT_dFileSize.Byte0 = SD_SPI_Read(tWithIncrement)
    FAT_dFileSize.Byte1 = SD_SPI_Read(tWithIncrement)
    FAT_dFileSize.Byte2 = SD_SPI_Read(tWithIncrement)
    FAT_dFileSize.Byte3 = SD_SPI_Read(tWithIncrement)

    FAT_bCurrentSectorInCluster = 0                         ' Reset the current sector number within the current cluster
    Dir_bResponse = 1
    Return
'----------------------------------------------------------------------------------------------------------
' Locate the sector to load
' Input     : BPB_bSectorsPerCluster holds the amount of sectors per cluster
'           : FAT_bCurrentSectorInCluster holds the current sector that is being used
' Output    : FAT_bCurrentSectorInCluster holds the current sector to be used
' Notes     : None

FAT_LocateSector:
    If FAT_bCurrentSectorInCluster = BPB_bSectorsPerCluster Then
        SD_PointToSectorWithBypass(FAT_wStartSector + File_wCurrentCluster.Byte1)   ' Point to the sector within the SD
        Sector_Skip_Bytes(File_wCurrentCluster.Byte0)                               ' \ Skip over that many entries (each entry is 16-bits)
        Sector_Skip_Bytes(File_wCurrentCluster.Byte0)                               ' /
        File_wCurrentCluster.Byte0 = SD_SPI_Read(tWithIncrement)                    ' \
        File_wCurrentCluster.Byte1 = SD_SPI_Read(tWithIncrement)                    ' / Get the new cluster number
        FAT_bCurrentSectorInCluster = 0                                             ' Reset the current sector number within the current cluster
    EndIf
    GoSub FAT_ConvertSectorToCluster                                                ' Find the working sector within the current cluster
    Inc FAT_bCurrentSectorInCluster
    Return
'----------------------------------------------------------------------------------------------------------
' Read a byte from the opened file
' Input     : None
' Output    : WREG contains the byte read from the file
' Notes     : None
'
$define File_ReadByte() _File_ReadByte

_File_ReadByte Macro- \Byte
    GoSub __File_ReadByte
    Return_Wreg
    Endm

#ifMacro- _File_ReadByte
__File_ReadByte:
    If File_dCurrentByteCount >= FAT_dFileSize Then             ' Have we reached the end of the file?
        FAT_bResponse = cFAT_EndOfFile                          ' Yes. So indicate an End Of File state
        tStartedReadingFile = False                             ' Reset the file reading start flag
        WREG = 0                                                ' Empty the WREG return value
        Return                                                  ' Exit the routine
    EndIf
    If tStartedReadingFile = False Then                         ' Are we busy reading a sector?
        GoSub FAT_LocateSector                                  ' No. So get the current sector indicated
        tStartedReadingFile = True                              ' Indicate that the sector has been pointed to and is ready to read
        Sector_wReadLoop = 512                                  ' 512 bytes per sector to read
    EndIf
    FAT_ByteIn = SD_SPI_Read(tWithIncrement)                    ' Read a byte from the current sector and increment up into it
    Inc File_dCurrentByteCount                                  ' Increment the overall byte count
    Dec Sector_wReadLoop                                        ' Decrement the byte counter
    If Sector_wReadLoop = 0 Then                                ' Has the sector word counter reached 0?
        tStartedReadingFile = False                             ' Yes. So reset the file reading start flag
    EndIf
    FAT_bResponse = 255
    WREG = FAT_ByteIn
    Return
#endIfMacro-
'----------------------------------------------------------------------------------------------------------
' Read a word (16-bits) from the opened file
' Input     : None
' Output    : PRODL\H contains the word read from the file
' Notes     : None
'
$define File_ReadWord() _File_ReadWord

_File_ReadWord Macro- \Word
    GoSub __File_ReadWord
    Return_Word FAT_WordIn
    Endm

#ifMacro- _File_ReadWord
__File_ReadWord:
    If File_dCurrentByteCount >= FAT_dFileSize Then             ' Have we reached th end of the file?
        FAT_bResponse = cFAT_EndOfFile                          ' Yes. So indicate an End Of File state
        tStartedReadingFile = False                             ' Reset the file reading start flag
        FAT_WordIn = 0                                          ' Empty the FAT_WordIn return value
        Return                                                  ' Exit the routine
    EndIf
    If tStartedReadingFile = False Then                         ' Are we busy reading a sector?
        GoSub FAT_LocateSector                                  ' No. So get the current sector indicated
        tStartedReadingFile = True                              ' Indicate that the sector has been pointed to and is ready to read
        Sector_wReadLoop = 512                                  ' 512 bytes per sector to read
    EndIf
    FAT_WordIn.Byte0 = SD_SPI_Read(tWithIncrement)              ' Read a byte from the current sector and increment up into it
    FAT_WordIn.Byte1 = SD_SPI_Read(tWithIncrement)              ' Read another byte from the current sector and increment up into it
    File_dCurrentByteCount = File_dCurrentByteCount + 2         ' Increment the overall word count
    Sector_wReadLoop = Sector_wReadLoop - 2                     ' Decrement the word counter
    If Sector_wReadLoop = 0 Then                                ' Has the sector word counter reached 0?
        tStartedReadingFile = False                             ' Yes. So reset the file reading start flag
    EndIf
    FAT_bResponse = 255
    Return
#endIfMacro-
'----------------------------------------------------------------------------------------------------------
_FAT_Main_:

