$ifndef _ILI9320_
$define _ILI9320_
'
' Amicus18 ILI9320 Colour Graphic LCD 8-bit interfacing library routines
' Written by Les Johnson. September 2011
'
Declare Warnings = Off
Declare Reminders = Off
    #disable Print, Cursor                  ' Remove the compiler's Print and cursor library subroutine

    Declare LCD_Type = Graphic              ' Inform the compiler that we're using a graphic LCD
    Declare LCD_X_Res = 240
    Declare LCD_Y_Res = 320

    Symbol cScreen_Hor_Size_M1 = __LCD_X_Res - 1
    Symbol cScreen_Ver_Size_M1 = __LCD_Y_Res - 1
'
' Define the pins used by the Colour LCD
' Defaults are for use with the Amicus Touch Screen Colour LCD shield
'
'$ifndef Glcd_Reset
'   $define Glcd_Reset PORTC.5              ' Connect to LCD's reset pin (Optional)
'$endif
$ifndef Glcd_CS
    $define Glcd_CS  PORTC.0                ' Connect to LCD's CS pin
$endif
$ifndef Glcd_RD
    $define Glcd_RD  PORTC.1                ' Connect to LCD's RD pin
$endif
$ifndef Glcd_RS
    $define Glcd_RS  PORTC.2                ' Connect to LCD's RS pin
$endif
$ifndef Glcd_WR
    $define Glcd_WR  PORTA.3                ' Connect to LCD's WR pin
$endif
$ifndef Glcd_Data
    $define Glcd_Data PORTB                 ' Connect to LCD's data bus
$endif

$ifndef Glcd_InitDelay
    $define Glcd_InitDelay 30               ' LCD initialise delay (in milliseconds)
$endif

$ifndef True
    $define True 1
$endif
$ifndef False
    $define False 0
$endif
'
' Pre-Defined colour codes
'                               RRRRRGGGGGGBBBBB
    Symbol clBlack             %0000000000000000
    Symbol clBrightBlue        %0000000000011111
    Symbol clBrightGreen       %0000011111100000
    Symbol clBrightCyan        %0000011111111111
    Symbol clBrightRed         %1111100000000000
    Symbol clBrightMagenta     %1111100000011111
    Symbol clBrightYellow      %1111111111100000
    Symbol clBlue              %0000000000010000
    Symbol clGreen             %0000010000000000
    Symbol clCyan              %0000010000010000
    Symbol clRed               %1000000000000000
    Symbol clMagenta           %1000000000010000
    Symbol clBrown             %1111110000000000
    Symbol clLightGray         %1000010000010000
    Symbol clDarkGray          %0100001000001000
    Symbol clLightBlue         %1000010000011111
    Symbol clLightGreen        %1000011111110000
    Symbol clLightCyan         %1000011111111111
    Symbol clLightRed          %1111110000010000
    Symbol clLightMagenta      %1111110000011111
    Symbol clYellow            %1111111111110000
    Symbol clWhite             %1111111111111111

    Symbol clGray0             %1110011100011100
    Symbol clGray1             %1100011000011000
    Symbol clGray2             %1010010100010100
    Symbol clGray3             %1000010000010000
    Symbol clGray4             %0110001100001100
    Symbol clGray5             %0100001000001000
    Symbol clGray6             %0010000100000100
'
' ILI9320 LCD chipset registers
'
    Symbol cStart_Osc               = $00       ' Start Oscillation
    Symbol cDriver_Output_Control   = $01       ' Driver Output Control
    Symbol cLCD_Driving_Control     = $02       ' LCD Driving Control
    Symbol cEntry_Mode              = $03       ' Entry Mode
    Symbol cResize_Control          = $04       ' Resize Control
    Symbol cDisplay_Control_1       = $07       ' Display Control 1
    Symbol cDisplay_Control_2       = $08       ' Display Control 2
    Symbol cDisplay_Control_3       = $09       ' Display Control 3
    Symbol cDisplay_Control_4       = $0A       ' Display Control 4
    Symbol cRGB_Disp_Interface_Control_1 = $0C  ' RGB Display Interface Control 1
    Symbol cFrame_Maker_Pos         = $0D       ' Frame Maker Position
    Symbol cRGB_Disp_Interface_Control_2 = $0F  ' RGB Display Interface Control 2
    Symbol cPower_Control_1         = $10       ' Power Control 1
    Symbol cPower_Control_2         = $11       ' Power Control 2
    Symbol cPower_Control_3         = $12       ' Power Control 3
    Symbol cPower_Control_4         = $13       ' Power Control 4

    Symbol cSet_Hor_GRAM_Address    = $20       ' Horizontal GRAM Address Set
    Symbol cSet_Ver_GRAM_Address    = $21       ' Vertical GRAM Address Set
    Symbol cAccess_CGRAM            = $22       ' Write or Read Data to/from CGRAM
    Symbol cPower_Control_7         = $29       ' Power Control 7
    Symbol cFrame_Color_Control     = $2B       ' Frame Rate and Color Control

    Symbol cGamma_Control_1         = $30       ' Gamma Control 1
    Symbol cGamma_Control_2         = $31       ' Gamma Control 2
    Symbol cGamma_Control_3         = $32       ' Gamma Control 3
    Symbol cGamma_Control_4         = $35       ' Gamma Control 4
    Symbol cGamma_Control_5         = $36       ' Gamma Control 5
    Symbol cGamma_Control_6         = $37       ' Gamma Control 6
    Symbol cGamma_Control_7         = $38       ' Gamma Control 7
    Symbol cGamma_Control_8         = $39       ' Gamma Control 8
    Symbol cGamma_Control_9         = $3C       ' Gamma Control 9
    Symbol cGamma_Control_10        = $3D       ' Gamma Control 10

    Symbol cHor_Address_Start_Pos   = $50       ' Horizontal Address Start Position
    Symbol cHor_Address_End_Pos     = $51       ' Horizontal Address End Position
    Symbol cVer_Address_Start_Pos   = $52       ' Vertical Address Start Position
    Symbol cVer_Address_End_Pos     = $53       ' Vertical Address End Position
    Symbol cGate_Scan_Control_1     = $60       ' Gate Scan Control 1
    Symbol cGate_Scan_Control_2     = $61       ' Gate Scan Control 2
    Symbol cGate_Scan_Control_3     = $6A       ' Gate Scan Control 3
    Symbol cPart_Image1_Display_Pos = $80       ' Partial Image 1 Display Position
    Symbol cPart_Image1_Area_Start  = $81       ' Partial Image 1 Area (Start Line)
    Symbol cPart_Image1_Area_End    = $82       ' Partial Image 1 Area (End Line)
    Symbol cPart_Image2_Display_Pos = $83       ' Partial Image 2 Display Position
    Symbol cPart_Image2_Area_Start  = $84       ' Partial Image 2 Area (Start Line)
    Symbol cPart_Image2_Area_End    = $85       ' Partial Image 2 Area (End Line)
'
' Variables
'
    Dim _Glcd_bFlags As Byte System             ' Used to store critical bits
    Dim tVerticalPrint As _Glcd_bFlags.0        ' True if the print routine is display characters vertically
    Dim tTransparent As _Glcd_bFlags.1          ' True if transparent text required. i.e. no paper colour
    Dim Glcd_bParamL As Byte System
    Dim Glcd_bParamH As Byte System
    Dim Glcd_bParamLH As Glcd_bParamH
    Dim Glcd_wParam As Glcd_bParamL.Word        ' Used to pass a single word sized parameter

    Dim wPixelColour As Word System             ' Holds the pixel's colour
    Dim wBackGroundColour As Word System        ' Holds the background colour for text printing
    Dim wPixelColor As wPixelColour             ' Holds the pixel's colour
    Dim wBackGroundColor As wBackGroundColour   ' Holds the background colour for text printing

    Dim _Glcd_Xpos As Byte System               ' Bring the compiler's system variable into the program
    Dim Glcd_bCursorX As _Glcd_Xpos             ' 8-bit Text cursor X position
    Dim Glcd_wCursorY As Word System            ' 16-bit Text cursor Y position

    Dim Glcd_wPixelXpos As Word System          ' 16-bit Pixel X position
    Dim Glcd_bPixelXpos As Glcd_wPixelXpos.Byte0 ' 8-bit Pixel X position (aliased)
    Dim Glcd_wPixelYpos As Word System          ' 16-bit Pixel Y position
'
' Glcd_Circle and Rectangle variables and aliases
'
    Dim Glcd_wQuadA As Word
    Dim Glcd_wQuadB As Word
    Dim Glcd_wQuadC As Word
    Dim Glcd_wQuadD As Word

    Dim Glcd_wXpos_S As Word
    Dim Glcd_bXpos_S As Glcd_wXpos_S.Byte0
    Dim Glcd_wYpos_S As Word

    Dim Glcd_wSize As Word                      ' 16-bit Size of a circle, or a line, or a box
    Dim Glcd_bSize As Glcd_wSize.Byte0          ' 8-bit Size of a circle, or a line, or a box
    Dim Glcd_wHeight As Word                    ' 16-bit Height value
    Dim Glcd_wLength As Word                    ' 16-bit Length value
    Dim Glcd_bLength As Glcd_wLength.Byte0
    Dim Glcd_bWidth As Glcd_wLength.Byte0

    Dim Glcd_wTR As Word
    Dim Glcd_wDD As Word

    Dim PP_bAddress As Glcd_wTR.Byte0
    Dim PP_bAddressH As Glcd_wTR.Byte1
    Dim PP_bAddressHH As Glcd_wDD.Byte0
    Dim PP_bAddressHHH As Glcd_wDD.Byte1
    Dim PP_dAddress As PP_bAddress.Dword

    Dim Glcd_wImageSize As PP_bAddress.Word     ' Holds the image size

    Dim Glcd_wXX As Word
    Dim Glcd_wYY As Word
'
' Used for Glcd_WriteReg
'
    Dim Glcd_bRegIndex As Glcd_wXX.Byte0
    Dim Glcd_bRegister As Glcd_wXX.Byte1
    Dim Glcd_wRegValue As Glcd_wParam
'
' Glcd_SetWindow aliases
'
    Dim Glcd_bX1 As Glcd_wXX.Byte0
    Dim Glcd_bX2 As Glcd_wXX.Byte1
    Dim Glcd_bY1 As Glcd_wYY
    Dim Glcd_bY2 As Glcd_wDD
'
' Glcd_Line variables aliased from circle variables
'
    Dim Glcd_wXpos_E As Glcd_wQuadA
    Dim Glcd_bXpos_E As Glcd_wXpos_E.Byte0
    Dim Glcd_wYpos_E As Glcd_wQuadB
    Dim Glcd_wDeltaY As Glcd_wQuadC
    Dim Glcd_wDeltaX As Glcd_wQuadD

    Dim Glcd_wStepY As Glcd_wXX
    Dim Glcd_wDeviation As Glcd_wTR
    Dim Glcd_wStepX As Glcd_wDD
'
' Character display variables
'
    Dim Glcd_wHeaderAddress As Word          ' Holds the address of the font offset table (in code memory)
    Dim Glcd_bFontAmountReads As Byte        ' Holds the amount of bytes per Y segment of the character's font
    Dim Glcd_bFontHeight As Byte             ' Holds the height of the character (in pixels)
    Dim Glcd_bFontWidth As Byte              ' Holds the width of the font (in pixels)
'
' Bring the system variable that the compiler's library Print would use into the program
'
    Dim GPR As Byte System
    Dim GEN3 As Byte System
    Dim GEN3H As Byte System
    Dim PP0 As Byte System
    Dim PP0H As Byte System
    Dim PP3 As Byte System
    Dim PP4 As Byte System
    Dim PP3H As Byte System
    Dim PP5 As Byte System
    Dim PP5H As Byte System
    Dim PP6 As Byte System
    Dim PP6H As Byte System
'
' Alias variables used in Glcd_PrintChar to system variables
'
    Dim Glcd_bSaveChar As PP3H
    Dim Glcd_bCharReadsTemp As PP0
    Dim Glcd_bCharPixelsTemp As PP0H
    Dim Glcd_bHeightCount As PP3
    Dim Glcd_bWidthCount As PP4
    Dim Glcd_wSaveTblPtr As Word           ' Save the contents of SFRs TBLPTRL\H
'
' Alias some SFRs as 16-bit variables
'
    Dim ILI_wTblPtr As TBLPTRL.Word
    Dim ILI_wProd As PRODL.Word
    Dim ILI_wFSR0 As FSR0L.Word
    Dim ILI_wFSR1 As FSR1L.Word
'
' Aliases to variables. Used in Image routines
'
    Dim Glcd_bMag As Glcd_wPixelXpos.Byte1
    Dim Glcd_wStartOfLine As ILI_wProd
    Dim Glcd_bDown As Glcd_wYY.Byte0
    Dim Glcd_bAcross As Glcd_wYY.Byte1
    Dim Glcd_bWinEndXPos As Glcd_wXX.Byte0
    Dim Glcd_wWinEndYPos As FSR0L.Word
    Dim Glcd_bPixelWidth As Glcd_wXX.Byte1
    Dim Glcd_bMultMinusOne As FSR1L

'------------------------------------------------------------------------------------------
    GoTo _ILI9320_Main_                     ' Jump over the subroutines
'------------------------------------------------------------------------------------------
' Load font/s into code memory
' All the fonts used by a program must be included here
'
'#ifMacro- _SetFont, _Glcd_PrintChar, _Glcd_Print
    'Include "Courier_Bold_14.inc"
'#endIfMacro-

'------------------------------------------------------------------------------------------
' Register and it's corresponding value for LCD setup
' These are placed into a code memory table for scanning by the LCD setup routine
' A register value of $FF will cause a millisecond delay
'
Glcd_RegisterValues:
    CData Byte $E5, Word $8000,_            ' Set the internal vcore voltage
          Byte $00, Word $0001,_            ' Start oscillator
          Byte $FF, Word 10,_               ' Delay 10ms
          Byte $A4, Word $0001,_            ' Calibrate
          Byte $07, Word $0000,_            ' Display control(1)
          Byte $FF, Word 10,_               ' Delay 10ms
         '
         ' Display Setting
         '
          Byte $01, Word $0100,_            ' Driver output control (1)
          Byte $02, Word $0700,_            ' LCD driving control
          Byte $03, Word $1030,_            ' Entry mode
                                            ' Bit-3: Address is updated in horizontal writing direction
                                            ' Bit-4, Bit-5: Auto increment vertical and horizontal
                                            ' Bit-9: High speed RAM disabled
                                            ' Bit-12: Swap the RGB data to BGR when writing into GRAM
          Byte $04, Word $0000,_            ' Resize control
          Byte $08, Word $0202,_            ' Display control (2)
          Byte $09, Word $0000,_            ' Display control (3)
          '
          ' Power Control
          '
          Byte $07, Word $0173,_            ' Display Control 1
          Byte $17, Word $0001,_
          Byte $10, Word $0000,_
          Byte $11, Word $0007,_            ' Power control 2 DC, VC
          Byte $12, Word $0000,_            ' Power control 3 VRH
          Byte $13, Word $0000,_            ' Power control 4 vcom amplitude
          Byte $FF, Word 20,_               ' Delay 20ms
          Byte $10, Word $16B0,_            ' Power control 1 BT,AP
          Byte $11, Word $0037,_            ' Power control 2 DC,VC
          Byte $FF, Word 50,_               ' Delay 50ms
          Byte $12, Word $013E,_            ' Power control 3 VRH
          Byte $FF, Word 50,_               ' Delay 50ms
          Byte $13, Word $1A00,_            ' Power control 4 vcom amplitude
          Byte $29, Word $000F,_            ' Power control 7 vcomH
          Byte $FF, Word 50,_               ' Delay 50ms
          Byte $20, Word $0000,_            ' Horizontal GRAM Address Set
          Byte $21, Word $0000,_            ' Vertical GRAM Address Set
          Byte $50, Word $0000,_            ' Horizontal Address Start Position
          Byte $51, Word $00EF,_            ' Horizontal Address end Position (239)
          Byte $52, Word $0000,_            ' Vertical Address Start Position
          Byte $53, Word $013F,_            ' Vertical Address end Position (319)
          Byte $60, Word $2700,_            ' Gate Scan Control 1 (Set bit-15 for reversed display position)
          Byte $61, Word $0003,_            ' Gate Scan Control 2 (Scroll enabled, positive display(bit-0 controls positive or negative))
          Byte $6A, Word $0000,_            ' Gate Scan Control 3
          Byte $90, Word $0010,_            ' Panel Interface Control 1
          Byte $92, Word $0000,_            ' Panel Interface Control 2
          Byte $93, Word $0000,_            ' Panel Interface Control 3
          '
          ' Gamma Control
          '
          Byte $30, Word $0007,_            ' Gamma Control 1
          Byte $31, Word $0403,_            ' Gamma Control 2
          Byte $32, Word $0404,_            ' Gamma Control 3
          Byte $35, Word $0002,_            ' Gamma Control 4
          Byte $36, Word $0707,_            ' Gamma Control 5
          Byte $37, Word $0606,_            ' Gamma Control 6
          Byte $38, Word $0106,_            ' Gamma Control 7
          Byte $39, Word $0007,_            ' Gamma Control 8
          Byte $3C, Word $0700,_            ' Gamma Control 9
          Byte $3D, Word $0707              ' Gamma Control 10

'-------------------------------------------------------------------------------------------------

(*********************************************************************
* Overview  : Helper routine that writes to the LCD's index register *
* Input     : WREG - index register                                  *
* Output    : None                                                   *
* Notes     : None                                                   *
**********************************************************************)
$define hSetIndex(pIndex) WREG = pIndex : _hSetIndex

_hSetIndex Macro-
    GoSub __hSetIndex
Endm

'#ifMacro- _hSetIndex, _hSetReg, _hSetCGRAM, _Glcd_SolidFill
__hSetIndex:
    Clear Glcd_RS
    Clear Glcd_Data             ' Write 00 to the LCD
    Clear Glcd_WR
    Set Glcd_WR
    Glcd_Data = WREG
    Clear Glcd_WR
    Set Glcd_WR
    Return
'#endIfMacro-

(*********************************************************************
* Overview  : Helper routine that writes 16-bit Data to the LCD      *
* Input     : Glcd_bParamH,Glcd_bParamL - data bytes                 *
* Output    : None                                                   *
* Notes     : Accepts data as two seperate bytes                     *
**********************************************************************)
$define hWriteData(pByte1,pByte0) '
    Glcd_bParamH = pByte1 '
    WREG = pByte0 '
    _hWriteData

_hWriteData Macro-
    GoSub __hWriteData
Endm

#ifMacro- _hWriteData, _hWriteDataWord, _hSetReg, _Glcd_ImageFromCode
__hWriteData:
    Glcd_bParamL = WREG                     ' Transfer WREG into its working variable
__hWriteData_P:                             ' Vector for hWriteData to call with Glcd_bParamL\H loaded
    Set Glcd_RS
    Glcd_Data = Glcd_bParamH
    Clear Glcd_WR
    Set Glcd_WR
    Glcd_Data = Glcd_bParamL
    Clear Glcd_WR
    Set Glcd_WR
    Return
#endIfMacro-

(*********************************************************************
* Overview  : Helper routine that writes 16-bit Data to the LCD      *
* Input     : Glcd_bParamH,Glcd_bParamL - data word                  *
* Output    : None                                                   *
* Notes     : Accepts data as a single word parameter                *
**********************************************************************)
$define hWriteDataWord(pWord) '
    Glcd_bParamH = pWord.Byte1 '
    WREG = pWord.Byte0 '
    _hWriteDataWord

_hWriteDataWord Macro-
    GoSub __hWriteData
Endm

(*********************************************************************
* Overview  : Helper routine that reads 16-bit data from the LCD     *
* Input     : None                                                   *
* Output    : Glcd_bParamH,Glcd_bParamL - Data bytes                 *
* Notes     : None                                                   *
**********************************************************************)
$define hReadData() _hReadData

_hReadData Macro-
    GoSub __hReadData
Endm

#ifMacro- _hReadData
__hReadData:
    Set Glcd_RS
    Glcd_bParamH = Glcd_Data
    Clear Glcd_RD
    Set Glcd_RD

    Clear Glcd_RD
    Glcd_bParamL = Glcd_Data
    Set Glcd_RD
    Clear Glcd_RS
    Return
#endIfMacro-

(***********************************************************************
* Overview  : Helper macro that writes an address pointer to the LCD   *
* Input     : pAddr0, pAddr1, pAddr2 - Address bytes                   *
* Output    : None                                                     *
* Notes     : None                                                     *
*                                                                      *
************************************************************************)
$define hSetAddress(pAddr2, pAddr1, pAddr0)  '
        hSetIndex(cSet_Hor_GRAM_Address): hWriteData(0, pAddr0) '
        hSetIndex(cSet_Ver_GRAM_Address): hWriteData(pAddr2, pAddr1) '
        hSetIndex(cAccess_CGRAM)

(***********************************************************************
* Overview  : Helper routine that sets graphics controller register    *
* Input     : Glcd_bParamL-Glcd_bParamH - register number              *
*           : WREG - value to be set                                   *
* Output    : None                                                     *
* Notes     : None                                                     *
************************************************************************)
$define hSetReg(pIndex, pValue) '
    Glcd_wParam = pValue '
    WREG = pIndex   '
    _hSetReg

_hSetReg Macro-
    GoSub __hSetReg
Endm

#ifMacro- _hSetReg
__hSetReg:
    GoSub __hSetIndex
    GoSub __hWriteData_P
    Return
#endIfMacro-
(***********************************************************************
* Overview  : Helper routine to set an address in CGRAM                *
* Input     : Glcd_wPixelYpos holds the Y position (0 to 319)          *
*           : WREG holds the X position (0 to 239)                     *
* Output    : None                                                     *
* Notes     : Does not point to CGRAM                                  *
************************************************************************)
$define hSetCGRAM(pXpos,pYpos) '
    Glcd_wPixelYpos = pYpos '
    WREG = pXpos '
    _hSetCGRAM

_hSetCGRAM Macro-
    GoSub __hSetCGRAM
Endm

#ifMacro- _hSetCGRAM, _Glcd_Vline, _Glcd_Hline
__hSetCGRAM:
    hSetReg($20, WREG)
    hSetReg($21, Glcd_wPixelYpos)
    hSetIndex($22)              ' Access CGRAM
    Return
#endIfMacro-
(***********************************************************************
* Overview  : Helper routine to fill a windowed area with a colour     *
* Input     : PP_dAddress holds the amount of pixels to write          *
*           : wPixelColour holds the pixel colour                      *
* Output    : None                                                     *
* Notes     : None                                                     *
************************************************************************)
$define hSolidFill(pCount) '
    PP_dAddress = pCount '
    _Glcd_SolidFill

_Glcd_SolidFill Macro-
    GoSub __Glcd_SolidFill
Endm

#ifMacro- _Glcd_SolidFill, _Glcd_SolidRectangle, _Glcd_Cls
__Glcd_SolidFill:
    If PP_dAddress = 0 Then Return
    Clear Glcd_CS           ' Enable the LCD
    hSetIndex($22)          ' Access CGRAM
    'hWriteData(wPixelColour.Byte1,wPixelColour.Byte0)
    Repeat
        hWriteData(wPixelColour.Byte1,wPixelColour.Byte0)
        Dec PP_dAddress
    Until PP_dAddress = 0
    Set Glcd_CS             ' Disable the LCD
    Return
#endIfMacro-
(*************************************************************************
* Overview  : Helper Macro that returns maximum horizontal coordinate    *
* Input     : None                                                       *
* Output    : maximum horizontal coordinate                              *
* Notes     : None                                                       *
**************************************************************************)
$define hGetMaxX() cScreen_Hor_Size_M1

(*************************************************************************
* Overview  : Helper Macro that returns maximum vertical coordinate      *
* Input     : None                                                       *
* Output    : maximum vertical coordinate                                *
* Notes     : None                                                       *
**************************************************************************)
$define hGetMaxY() cScreen_Ver_Size_M1
(***********************************************************************
* Write to an LCD register                                             *
* Input     : WREG holds the register number                           *
*             Glcd_wParam holds the 16-bit data to write               *
* Output    : None                                                     *
* Notes     :                                                          *
************************************************************************)
$define Glcd_WriteReg(pRegister, pValue) '
    Glcd_wParam = pValue '
    WREG = pRegister     '
    _Glcd_WriteReg

_Glcd_WriteReg Macro-
    GoSub __Glcd_WriteReg
    Endm

#ifMacro- _Glcd_WriteReg, _hScroll_, _Glcd_ScrollUp, _Glcd_ScrollDown
__Glcd_WriteReg:
    Clear Glcd_CS           ' Enable the LCD
    _hSetReg                ' Call the macro
    Set Glcd_CS             ' Disable the LCD
    Return
#endIfMacro-

(***********************************************************************
* Overview  : Reset and initialise the LCD                             *
* Input     : None                                                     *
* Output    : None                                                     *
* Notes     : None                                                     *
************************************************************************)
$define Glcd_Init() _ResetDevice

_ResetDevice Macro-
    GoSub sResetDevice
Endm

#ifMacro- _ResetDevice
sResetDevice:
    tVerticalPrint = False      ' Disable vertical printing
    tTransparent = False        ' Clear the transparency bit (use paper colour for text)
    High Glcd_CS                ' Disable the LCD
$ifdef Glcd_Reset               ' Is the interface using the LCD's reset pin?
    Low Glcd_Reset              ' Yes. So Hold the LCD in reset
$endif
    Low Glcd_RS                 ' Set RS pin as output low
    High Glcd_RD                ' Set RD pin as output high
    High Glcd_WR                ' Set WR pin as output high
    Output Glcd_Data            ' Set Data Bus as output

$ifdef Glcd_Reset               ' Is the interface using the LCD's reset pin?
    Clear Glcd_Reset            ' \
    DelayMS 2                   ' |
    Set Glcd_Reset              ' | Yes. So reset LCD's controller
    DelayMS 2                   ' /
$endif
'
' Setup display for the ILI9320 driver IC
'
    Clear Glcd_CS               ' Enable the LCD
    ILI_wTblPtr = VarPtr Glcd_RegisterValues ' Point TBLPTRL\H to the start of the register table
    For Glcd_bRegIndex = 48 To 0 Step -1    ' Create a loop for 49 registers(and or delays)
        Tblrd*+                             ' Read the register number from the table
        Glcd_bRegister = TABLAT             ' Place it into Glcd_bRegister
        Tblrd*+                             ' Read the low byte of the 16-bit value to place into the register
        Glcd_wRegValue.Byte0 = TABLAT       ' Place it into Glcd_wRegValue.Byte0
        Tblrd*+                             ' Read the high byte of the 16-bit value to place into the register
        Glcd_wRegValue.Byte1 = TABLAT       ' Place it into Glcd_wRegValue.Byte1
        If Glcd_bRegister = $FF Then        ' Was the register number $FF?
            DelayMS Glcd_wRegValue.Byte0    ' Yes. So this was a millisecond delay
        Else                                ' Otherwise...
            hSetReg(Glcd_bRegister,Glcd_wRegValue) ' Send the register and its value to the LCD
        EndIf
    Next
    Set Glcd_CS                 ' Disable the LCD
    DelayMS 30                  ' Delay before returning
    Return
#endIfMacro-
(*************************************************************************
* Overview  : Converts true color into 5:6:5 RGB format.                 *
* Input     : Red, Green, Blue components.                               *
* Output    : 5 bits red, 6 bits green, 5 bits blue color.               *
* Notes     : None                                                       *
**************************************************************************)
$define hRGB565Convert(pRed, pGreen, pBlue) ((((pRed) >> 3) << 11) | (((pGreen) >> 2) << 5) | ((pBlue) >> 3))

(*************************************************************************
* Overview  : Helper Macro that returns the green value of the pixel     *
* Input     : 16-bit Colour coded in format:                             *
*                   bits 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00 *
*               pColour  R  R  R  R  R  G  G  G  G  G  G  B  B  B  B  B  *
* Output    : Returns the 5-bit red value                                *
* Notes     : None                                                       *
**************************************************************************)
$define hGetColorRed(pColour, pResult) '
    Glcd_wParam = pColour     '
    _hGetColorRed             '
    pResult = WREG

_hGetColorRed Macro-
    GoSub __hGetColorRed
Endm

#ifMacro- _hGetColorRed
__hGetColorRed:
    WREG = Glcd_bParamH     ' Get the high byte of the pixel's colour as this contains the red content
    WREG = WREG >> 3        ' Shift it into position
    Return
#endIfMacro-
(*************************************************************************
* Overview  : Helper Macro that returns the green value of the pixel     *
* Input     : 16-bit Colour coded in format:                             *
*                   bits 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00 *
*               pColour  R  R  R  R  R  G  G  G  G  G  G  B  B  B  B  B  *
* Output    : Returns the 6-bit green value                              *
* Notes     : None                                                       *
**************************************************************************)
$define hGetColorGreen(pColour, pResult) '
    Glcd_wParam = pColour       '
    _hGetColorGreen             '
    pResult = WREG

_hGetColorGreen Macro-
    GoSub __hGetColorGreen
Endm

#ifMacro- _hGetColorGreen
__hGetColorGreen:
    Glcd_bParamH = Glcd_bParamH << 3  ' Get the High Byte of the Pixel's colour as this contains some of the green content
    Glcd_bParamH = Glcd_bParamH & %00111000
    Glcd_bParamL = Glcd_bParamL >> 5  ' Get the Low Byte of the Pixel's colour as this contains some of the green content
    WREG = Glcd_bParamH | Glcd_bParamL
    Return
#endIfMacro-
(*************************************************************************
* Overview  : Helper Macro that returns the blue value of the pixel      *
* Input     : 16-Bit Colour coded in format:                             *
*                   bits 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00 *
*               pColour  R  R  R  R  R  G  G  G  G  G  G  B  B  B  B  B  *
* Output    : Returns the 5-bit blue value                               *
* Notes     : None                                                       *
**************************************************************************)
$define hGetColorBlue(pColour, pResult) '
    WREG = pColour             '
    _hGetColorBlue             '
    pResult = WREG

_hGetColorBlue Macro-
    GoSub __hGetColorBlue
Endm

#ifMacro- _hGetColorBlue
__hGetColorBlue:
    WREG = WREG & %00011111  ' Get the low Byte of the Pixel's colour as this contains some of the blue content
    Return
#endIfMacro-
(*************************************************************************
* Overview  : Helper Macro that sets current colour                      *
* Input     : wPixelColour coded in format:                              *
*                   bits 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00 *
*               pColour  R  R  R  R  R  G  G  G  G  G  G  B  B  B  B  B  *
* Output    : None                                                       *
* Notes     : None                                                       *
**************************************************************************)
$define hSetColour(pColour) wPixelColour = pColour
$define hSetColor(pColour) wPixelColour = pColour

(*************************************************************************
* Overview  : Helper Macro that sets current colour in RGB               *
* Input     : wPixelColour coded in format:                              *
*                   bits 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00 *
*               pColour  R  R  R  R  R  G  G  G  G  G  G  B  B  B  B  B  *
* Output    : None                                                       *
* Notes     : None                                                       *
**************************************************************************)
$define hSetColorRGB(pRed, pGreen, pBlue) '
Asm-                                          '
 Variable aRGB = ((pRed & 0x1F) << 11), aGreen = ((pGreen & 0x3F) << 4), aBlue = (pBlue & 0x1F), aRGB = aRed + aGreen + aBlue '
Endasm-                                       '
    Movlw (aRGB & 255)                        '
    Movwf wPixelColour.Byte0                  '
    Movlw (aRGB >> 8)                         '
    Movwf wPixelColour.Byte1

$define hSetColourRGB(pRed, pGreen, pBlue) hSetColorRGB(pRed, pGreen, pBlue)
(*************************************************************************
* Overview  : Helper Macro that sets current Red content                 *
* Input     : wPixelColour coded in format:                              *
*                   bits 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00 *
*               pColour  R  R  R  R  R  G  G  G  G  G  G  B  B  B  B  B  *
* Output    : None                                                       *
* Notes     : None                                                       *
**************************************************************************)
$define hSetColourRed(pRed) hSetColorRed(pRed)

$define hSetColorRed(pRed) '
    WREG = pRed '
    _hSetColorRed

_hSetColorRed Macro-
    GoSub __hSetColorRed
Endm

#ifMacro- _hSetColorRed
__hSetColorRed:
    Glcd_bParamL = WREG
    wPixelColour.Byte1 = wPixelColour.Byte1 & %00000111
    WREG = Glcd_bParamL & %00011111
    WREG = WREG << 3
    wPixelColour.Byte1 = wPixelColour.Byte1 | WREG
    Return
#endIfMacro-

(*************************************************************************
* Overview  : Helper Macro that sets current Green content               *
* Input     : wPixelColour coded in format:                              *
*                   bits 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00 *
*               pColour  R  R  R  R  R  G  G  G  G  G  G  B  B  B  B  B  *
* Output    : None                                                       *
* Notes     : None                                                       *
**************************************************************************)
$define hSetColourGreen(pGreen) hSetColorGreen(pGreen)

$define hSetColorGreen(pGreen) '
    WREG = pGreen              '
    _hSetColorGreen

_hSetColorGreen Macro-
    GoSub __hSetColorGreen
Endm

#ifMacro- _hSetColorGreen
__hSetColorGreen:
    PRODL = WREG
    wPixelColour = wPixelColour & %1111100000011111
    Glcd_wParam = ((PRODL & %00111111) << 5)
    wPixelColour = wPixelColour | Glcd_wParam
    Return
#endIfMacro-

(*************************************************************************
* Overview  : Helper Macro that sets current Blue content                *
* Input     : wPixelColour coded in format:                              *
*                   bits 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00 *
*               pColour  R  R  R  R  R  G  G  G  G  G  G  B  B  B  B  B  *
* Output    : None                                                       *
* Notes     : None                                                       *
**************************************************************************)
$define hSetColourBlue(pBlue) hSetColorBlue(pBlue)

$define hSetColorBlue(pBlue) '
    WREG = pBlue             '
    _hSetColorBlue

_hSetColorBlue Macro-
    GoSub __hSetColorBlue
Endm

#ifMacro- _hSetColorBlue
__hSetColorBlue:
    Glcd_bParamL = WREG
    wPixelColour.Byte0 = wPixelColour.Byte0 & %11100000
    WREG = Glcd_bParamL & %00011111
    wPixelColour.Byte0 = wPixelColour.Byte0 | WREG
    Return
#endIfMacro-

(*************************************************************************
* Overview  : Helper Macro that returns the current pixel colour         *
* Input     : None                                                       *
* Output    : Colour coded in format:                                    *
*                   bits 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00 *
*               pColour  R  R  R  R  R  G  G  G  G  G  G  B  B  B  B  B  *
* Notes     : None                                                       *
**************************************************************************)
$define Glcd_GetPenColour() wPixelColour
$define Glcd_GetPenColor() wPixelColour

(*************************************************************************
* Overview  : Helper Macro that returns the current paper colour         *
* Input     : None                                                       *
* Output    : Colour coded in format:                                    *
*                   bits 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00 *
*               pColour  R  R  R  R  R  G  G  G  G  G  G  B  B  B  B  B  *
* Notes     : None                                                       *
**************************************************************************)
$define Glcd_GetPaperColour() wBackGroundColour
$define Glcd_GetPaperColor() wBackGroundColour

(*************************************************************************
* Overview  : Set current colour of a pixel                              *
* Input     : wPixelColour coded in format:                              *
*                   bits 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00 *
*               pColour  R  R  R  R  R  G  G  G  G  G  G  B  B  B  B  B  *
* Output    : None                                                       *
* Notes     : None                                                       *
**************************************************************************)
$define Glcd_PenColour(pColour) wPixelColour = pColour
$define Glcd_PenColor(pColour) wPixelColour = pColour

(*************************************************************************
* Overview  : Set current colour of the background                       *
* Input     : wBackGroundColour coded in format:                         *
*                   bits 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00 *
*               pColour  R  R  R  R  R  G  G  G  G  G  G  B  B  B  B  B  *
* Output    : None                                                       *
* Notes     : None                                                       *
**************************************************************************)
$define Glcd_PaperColour(pColour) wBackGroundColour = pColour
$define Glcd_PaperColor(pColour) wBackGroundColour = pColour

'-------------------------------------------------------------------------------------------------
' Overview  : Place a pixel on the LCD
' Input     : Glcd_bPixelXpos, Glcd_wPixelYpos = pixel coordinates
'           : wPixelColour = pixel colour
' Output    : None
' Notes     : None
'
$define Glcd_Plot(pXpos, pYpos) '
    Glcd_wPixelYpos = pYpos '
    WREG = pXpos '
    _PutPixel

_PutPixel Macro-
    GoSub __PutPixel
Endm

#ifMacro- _PutPixel, _Glcd_PrintChar, _Glcd_Print
__PutPixel:
    Glcd_bPixelXpos = WREG
'
' Check boundaries
'
    If Glcd_wPixelYpos.15 = 1 Then Return               ' Exit if the Ypos is less than 0
    If Glcd_bPixelXpos >= __LCD_X_Res Then Return       ' Exit if the Xpos is greater than the LCD's X resolution
    If Glcd_wPixelYpos >= __LCD_Y_Res Then Return       ' Exit if the Ypos is greater than the LCD's Y resolution

    Clear Glcd_CS                                       ' Enable the LCD's interface
    hSetCGRAM(Glcd_bPixelXpos,Glcd_wPixelYpos)          ' Point to the correct location in CGRAM
    hWriteData(wPixelColour.Byte1,wPixelColour.Byte0)   ' Write the pixel to the LCD
    Set Glcd_CS                                         ' Disable the LCD's interface
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Return a Pixel's colour from x,y position
' Input     : Glcd_bPixelXpos, Glcd_wPixelYpos - pixel coordinates
' Output    : Glcd_bParamL/H hold Pixel color
' Notes     : The pixel read from the LCD is reversed. i.e. BGR instead of RGB
'           : This is corrected to RGB before returning
'
$define Glcd_Pixel(pXpos, pYpos) _Glcd_Pixel pXpos,pYpos

_Glcd_Pixel Macro- pXpos, pYpos \Word
    #if (Prm_2 == Byte)
        Byte_Word pYpos,Glcd_wPixelYpos
    #endif
    #if (Prm_2 == Word) || (Prm_2 == Dword)
        Word_Word pYpos,Glcd_wPixelYpos
    #endif
    #if (Prm_2 == Num8) || (Prm_2 == Num16) || (Prm_2 == Num32)
        Num_Word pYpos,Glcd_wPixelYpos
    #endif

    #if (Prm_1 == Byte) || (Prm_1 == Word) || (Prm_1 == Dword)
        Byte_Wreg pXpos
    #else
        Num_Wreg pXpos
    #endif
    GoSub __Glcd_Pixel
    Return_Word Glcd_wParam
Endm

#ifMacro- _Glcd_Pixel
__Glcd_Pixel:
    Glcd_bPixelXpos = WREG
    Clear Glcd_CS                           ' Enable the LCD's interface
'
' Setup the Address of the pixel
'
    hSetCGRAM(Glcd_bPixelXpos,Glcd_wPixelYpos)  ' Point to the correct location in CGRAM

    Set Glcd_RS                             ' Place the LCD into control mode
    Input Glcd_Data                         ' Make the data port an input
    Clear Glcd_RD                           ' \ Read cycle to remove register value from data latch
    Set Glcd_RD                             ' /
    Clear Glcd_RD                           ' \ First Read cycle to move data from GRAM to Read Data Latch
    Set Glcd_RD                             ' /
    Clear Glcd_RD                           ' \ Second Read cycle to move data from GRAM to Read Data Latch
    Set Glcd_RD                             ' /
    Clear Glcd_RD                           ' \
    PRODH = Glcd_Data                       ' | First Read cycle to get high data from Read Data Latch
    Set Glcd_RD                             ' /
    Clear Glcd_RD                           ' \
    PRODL = Glcd_Data                       ' | Second Read cycle to get low data from Read Data Latch
    Set Glcd_RD                             ' /
'
' Reverse the bits of ILI_wProd into Glcd_wParam
'
    WREG = 16
__Glcd_PixelRevLoop:
    Rol ILI_wProd
    Ror Glcd_wParam
    Djnz WREG,__Glcd_PixelRevLoop
    Set Glcd_CS                             ' Disable the LCD's interface
    Output Glcd_Data                        ' Make the data port back to an output
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Draw a Circle on the Graphic LCD
' Input     : Glcd_wXpos_S holds the starting X position
'           : Glcd_wYpos_S holds the starting Y position
'           : Glcd_wSize holds the radius of the circle
' Output    : None
' Notes     : None
'
$define Glcd_Circle(pXpos, pYpos, pRadius) '
    Glcd_wYpos_S = pYpos '
    Glcd_wSize = pRadius '
    WREG = pXpos '
    _Glcd_Circle

_Glcd_Circle Macro-
    GoSub __Glcd_Circle
Endm

#ifMacro- _Glcd_Circle                              ' Add this block only if the macro has been used in the program
__Glcd_Circle:
    Glcd_wXpos_S = WREG                             ' Transfer WREG into its working variable
    If Glcd_wSize = 0 Then Return                   ' Trap a radius of 0
    Glcd_wDD = Glcd_wYpos_S - Glcd_wXpos_S
    Glcd_wXX = 0
    Glcd_wYY = Glcd_wSize
    Glcd_wTR = 3 - (2 * Glcd_wSize)
    While Glcd_wXX <= Glcd_wYY
        Glcd_wQuadA = Glcd_wXpos_S + Glcd_wXX
        Glcd_wQuadB = Glcd_wYpos_S + Glcd_wYY

        Glcd_wQuadC = Glcd_wXpos_S - Glcd_wXX
        Glcd_wQuadD = Glcd_wYpos_S - Glcd_wYY
        GoSub _PlotIt
        Glcd_wQuadA = Glcd_wYpos_S + Glcd_wYY
        Glcd_wQuadA = Glcd_wQuadA - Glcd_wDD

        Glcd_wQuadB = Glcd_wYpos_S + Glcd_wXX
        Glcd_wQuadC = Glcd_wYpos_S - Glcd_wYY

        Glcd_wQuadC = Glcd_wQuadC - Glcd_wDD
        Glcd_wQuadD = Glcd_wYpos_S - Glcd_wXX
        GoSub _PlotIt
        If Glcd_wTR.15 = 1 Then
            Glcd_wTR = Glcd_wTR + 6
            Glcd_wTR = Glcd_wTR + (Glcd_wXX * 4)
        Else
            Glcd_wTR = Glcd_wTR + 10
            Glcd_wTR = Glcd_wTR + ((Glcd_wXX - Glcd_wYY) * 4)
            Dec Glcd_wYY
        EndIf
        Inc Glcd_wXX
    Wend
    Return
'
' Plot 4 locations simultaneously
' This is part of the Glcd_Circle subroutine
'
_PlotIt:
    Glcd_Plot(Glcd_wQuadA, Glcd_wQuadB)
    Glcd_Plot(Glcd_wQuadA, Glcd_wQuadD)
    Glcd_Plot(Glcd_wQuadC, Glcd_wQuadB)
    Glcd_Plot(Glcd_wQuadC, Glcd_wQuadD)
    Return
#endIfMacro-
'-------------------------------------------------------------------------------------------------
' Draw or Erase a line from Glcd_wXpos_S, Glcd_wYpos_S to Glcd_wXpos_E, Glcd_wYpos_E
' Input     : Glcd_wXpos_S = Starting X position of the Line
'           : Glcd_wYpos_S = Starting Y position of the Line
'           : Glcd_wXpos_E = Ending X position of the Line
'           : Glcd_wYpos_E = Ending Y position of the Line
' Output    : None
' Notes     :
'
$define Glcd_Line(pStartXpos, pStartYpos, pEndXpos, pEndYpos) '
    Glcd_wYpos_S = pStartYpos '
    Glcd_wXpos_E = pEndXpos '
    Glcd_wYpos_E = pEndYpos '
    WREG = pStartXpos '
    _Glcd_Line

_Glcd_Line Macro-
    GoSub __Glcd_Line
Endm

#ifMacro- _Glcd_Line, _Glcd_LineTo, _Glcd_Box, _Glcd_Rectangle      ' Add this block only if the macros have been used in the program
__Glcd_Line:
    Glcd_wXpos_S = WREG                                             ' Transfer WREG into its working variable
__Glcd_LineP:
    Glcd_wDeltaY = Glcd_wYpos_E - Glcd_wYpos_S                      ' Find the initial Y direction
    Glcd_wDeltaX = Glcd_wXpos_E - Glcd_wXpos_S                      ' Find the initial X direction

    Glcd_wStepY = 1                                                 ' Default to a positive step for Ypos
    If Glcd_wDeltaY.15 = 1 Then                                     ' Check if negative
        Glcd_wDeltaY = -Glcd_wDeltaY                                ' Negate Glcd_wDeltaY
        Glcd_wStepY = -1                                            ' And indicate a negative step
    EndIf
    Glcd_wStepX = 1                                                 ' Default to a positive step for Xpos
    If Glcd_wDeltaX.15 = 1 Then                                     ' Check if negative
        Glcd_wDeltaX = -Glcd_wDeltaX                                ' Negate Glcd_wDeltaX
        Glcd_wStepX = -1                                            ' And indicate a negative step
    EndIf
    Glcd_Plot(Glcd_wXpos_S,Glcd_wYpos_S)                            ' The first pixel plot is a special case
    If Glcd_wDeltaX > Glcd_wDeltaY Then
        Glcd_wDeviation = Glcd_wDeltaY - Glcd_wDeltaX
        While Glcd_wXpos_S <> Glcd_wXpos_E
            If Glcd_wDeviation.15 = 0 Then                          ' Check if negative
                Glcd_wYpos_S = Glcd_wYpos_S + Glcd_wStepY           ' Move on the Y axis directed by Glcd_wStepY
                Glcd_wDeviation = Glcd_wDeviation - Glcd_wDeltaX
            EndIf
            Glcd_wXpos_S = Glcd_wXpos_S + Glcd_wStepX               ' Move on the X axis directed by Glcd_wStepX
            Glcd_wDeviation = Glcd_wDeviation + Glcd_wDeltaY
            Glcd_Plot(Glcd_wXpos_S,Glcd_wYpos_S)                    ' Draw or Erase the pixel at this position
        Wend
        Return                                                      ' Return prematurely (this saves an else command)
    EndIf
    Glcd_wDeviation = Glcd_wDeltaX - Glcd_wDeltaY

    While Glcd_wYpos_S <> Glcd_wYpos_E
        If Glcd_wDeviation.15 = 0 Then                              ' Check if negative
            Glcd_wXpos_S = Glcd_wXpos_S + Glcd_wStepX               ' Move on the X axis directed by Glcd_wStepX
            Glcd_wDeviation = Glcd_wDeviation - Glcd_wDeltaY
        EndIf
        Glcd_wYpos_S = Glcd_wYpos_S + Glcd_wStepY                   ' Move on the Y axis directed by Glcd_wStepY
        Glcd_wDeviation = Glcd_wDeviation + Glcd_wDeltaX
        Glcd_Plot(Glcd_bXpos_S,Glcd_wYpos_S)                        ' Draw or Erase the pixel at this position
    Wend
    Return
#endIfMacro-
'-------------------------------------------------------------------------------------------------
' Draw a line from the previous end X and Y positions to new X and Y coordinates
' Input     : Glcd_wXpos_E holds the X position end
'           : ILI_wProd holds the Y position end
' Output    : None
' Notes     : None
'
$define Glcd_LineTo(pEndXpos,pEndYpos) '
    ILI_wProd = pEndYpos '
    WREG = pEndXpos '
    _Glcd_LineTo

_Glcd_LineTo Macro-
    GoSub __Glcd_LineTo
Endm

#ifMacro- _Glcd_LineTo              ' Add this block only if the macro has been used in the program
__Glcd_LineTo:
    Glcd_wXpos_S = Glcd_bXpos_E     ' Transfer previous X pos end to X pos start
    Glcd_wYpos_S = Glcd_wYpos_E     ' Transfer the previous Y pos to the Y pos start
    Glcd_wXpos_E = WREG             ' Transfer the new end X position
    Glcd_wYpos_E = ILI_wProd        ' Transfer the new end Y position
    GoSub __Glcd_LineP              ' Draw a line
    Return
#endIfMacro-
'-------------------------------------------------------------------------------------------------
' Create a window on the screen
' Input     : Glcd_bX1 holds the X position start (0 to 239)
'           : Glcd_bX2 holds the X position end (0 to 239)
'           : Glcd_bY1 holds the Y position start (0 to 319)
'           : Glcd_bY2 holds the Y position end (0 to 319)
' Output    : None
' Notes     : None
'
$define Glcd_SetWindow(pXstart, pYstart, pXend, pYend) '
    Glcd_bX2 = pXend   '
    Glcd_bY1 = pYstart '
    Glcd_bY2 = pYend   '
    WREG = pXstart     '
    _Glcd_SetWindow

_Glcd_SetWindow Macro-
    GoSub __Glcd_SetWindow
Endm

#ifMacro- _Glcd_SetWindow, _Glcd_Vline, _Glcd_Hline
__Glcd_SetWindow:
    Clear Glcd_CS               ' Enable the LCD
    hSetReg($50, WREG)          ' Horizontal Address Start Position (0 to 239)
    hSetReg($52, Glcd_bY1)      ' Vertical Address Start Position (0 to 319)
    hSetReg($51, Glcd_bX2)      ' Horizontal Address end Position (0 to 239)
    hSetReg($53, Glcd_bY2)      ' Vertical Address end Position (0 to 319)
    Set Glcd_CS                 ' Disable the LCD
    Return
 #endIfMacro-
'-------------------------------------------------------------------------------------------------
' Reset the window back to full screen
' Input     : None
' Output    : None
' Notes     : None
'
$define Glcd_ResetWindow() _Glcd_ResetWindow

_Glcd_ResetWindow Macro-
    GoSub __Glcd_ResetWindow
Endm

#ifMacro- _Glcd_ResetWindow, _Glcd_SolidRectangle
__Glcd_ResetWindow:
    Clear Glcd_CS               ' Enable the LCD
    hSetReg($50, $0000)         ' Horizontal Address Start Position
    hSetReg($52, $0000)         ' Vertical Address Start Position
    hSetReg($51, $00EF)         ' Horizontal Address end Position (239)
    hSetReg($53, $013F)         ' Vertical Address end Position (319)
    Set Glcd_CS                 ' Disable the LCD
    Return
 #endIfMacro-
'-------------------------------------------------------------------------------------------------
' Draw a vertical line
' Input     : Glcd_wPixelXpos = Starting X position of the Line
'           : Glcd_wPixelYpos = Starting Y position of the Line
'           : Glcd_wSize holds the length of the line (1 to 320)
' Output    : None
' Notes     : Uses a window and pixel blit method for speed
'
$define Glcd_Vline(pXpos, pYpos, pLength) '
    Glcd_wPixelYpos = pYpos '
    Glcd_wSize = pLength '
    WREG = pXpos '
    _Glcd_Vline

_Glcd_Vline Macro-
    GoSub __Glcd_Vline
Endm

#ifMacro- _Glcd_Vline                           ' Add this block only if the macro has been used in the program
__Glcd_Vline:
    Glcd_bPixelXpos = WREG                      ' Transfer WREG into its working variable
    If Glcd_wSize = 0 Then Return               ' Exit if the line length is 0
    If Glcd_bPixelXpos > 240 Then Return        ' Exit if X boundary is exceeded
    If Glcd_wPixelYpos > 320 Then Return        ' Exit if Y boundary is exceeded
    If Glcd_wSize > 320 Then Return             ' Exit if length is exceeded
    Glcd_SetWindow(Glcd_bPixelXpos, Glcd_wPixelYpos, Glcd_bPixelXpos, (Glcd_wPixelYpos + Glcd_wSize)) ' Create a window the size of the line required
    Clear Glcd_CS                               ' Enable the LCD
    hSetCGRAM(Glcd_bPixelXpos,Glcd_wPixelYpos)  ' Move to the location within CGRAM of the window
    Repeat
        Dec Glcd_wSize
        hWriteData(wPixelColour.Byte1,wPixelColour.Byte0)
    Until Glcd_wSize = 0
    Set Glcd_CS                                 ' Disable the LCD
    Glcd_ResetWindow()                          ' Reset the window to full size
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Draw a horizontal line
' Input     : Glcd_bPixelXpos = Starting X position of the Line
'           : Glcd_wPixelYpos = Starting Y position of the Line
'           : Glcd_wSize.Byte0 holds the length of the line (1 to 240)
' Output    : None
' Notes     : Uses a window And Pixel blit method For speed
'
$define Glcd_Hline(pXpos, pYpos, pLength) '
    Glcd_wPixelYpos = pYpos '
    Glcd_wSize.Byte0 = pLength '
    WREG = pXpos '
    _Glcd_Hline

_Glcd_Hline Macro-
    GoSub __Glcd_Hline
Endm

#ifMacro- _Glcd_Hline                           ' Add this block only if the macro has been used in the program
__Glcd_Hline:
    Glcd_bPixelXpos = WREG                      ' Transfer WREG into its working variable
    If Glcd_wPixelYpos > 320 Then Return        ' Exit if Y boundary is exceeded
    If Glcd_wSize.Byte0 = 0 Then Return         ' Exit if the line length is 0
    If Byte Glcd_bPixelXpos + Glcd_wSize.Byte0 > 240 Then Return ' Exit if X length is exceeded
    Glcd_SetWindow(Glcd_bPixelXpos, Glcd_wPixelYpos, (Glcd_bPixelXpos + Glcd_wSize.Byte0), Glcd_wPixelYpos) ' Create a window the size of the line required
    Clear Glcd_CS                               ' Enable the LCD
    hSetCGRAM(Glcd_bPixelXpos,Glcd_wPixelYpos)  ' Move to the location within CGRAM of the window
    Inc Glcd_wSize.Byte0
    Repeat
        Dec Glcd_wSize.Byte0
        hWriteData(wPixelColour.Byte1,wPixelColour.Byte0)
    Until Glcd_wSize.Byte0 = 0
    Set Glcd_CS                                 ' Disable the LCD
    Glcd_ResetWindow()                          ' Reset the window to full size
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Draw a square
' Input     : Glcd_wXpos_S holds the starting X position
'           : Glcd_wYpos_S holds the starting Y position
'           : Glcd_wSize holds the size of the square
' Output    : None
' Notes     :
'
$define Glcd_Box(pXpos, pYpos, pSize) '
    Glcd_wYpos_S = pYpos '
    Glcd_wSize = pSize '
    WREG = pXpos '
    _Glcd_Box

_Glcd_Box Macro-
    GoSub __Glcd_Box
Endm

#ifMacro- _Glcd_Box                         ' Add this block only if the macro has been used in the program
__Glcd_Box:
    Glcd_wXpos_S = WREG                     ' Transfer WREG into its working variable
    Glcd_Line(Glcd_wXpos_S, Glcd_wYpos_S, Glcd_wXpos_S + Glcd_wSize, Glcd_wYpos_S)  ' Draw top line
    Glcd_wYpos_S = Glcd_wYpos_E             ' Transfer previous Y pos end to Y pos start
    Glcd_wXpos_S = Glcd_bXpos_E             ' Transfer previous X pos end to X pos start
    Glcd_wYpos_E = Glcd_wYpos_E + Glcd_wSize
    GoSub __Glcd_LineP                      ' Draw right line

    Glcd_wYpos_S = Glcd_wYpos_E             ' Transfer previous Y pos end to Y pos start
    Glcd_wXpos_S = Glcd_bXpos_E             ' Transfer previous X pos end to X pos start
    Glcd_wXpos_E = Glcd_wXpos_E - Glcd_wSize
    GoSub __Glcd_LineP                      ' Draw bottom line

    Glcd_wYpos_S = Glcd_wYpos_E             ' Transfer previous Y pos end to Y pos start
    Glcd_wXpos_S = Glcd_bXpos_E             ' Transfer previous X pos end to X pos start
    Glcd_wYpos_E = Glcd_wYpos_E - Glcd_wSize
    GoSub __Glcd_LineP                      ' Draw left line
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Draw a Rectangle
' Input     : Glcd_wXpos_S holds the starting X position
'           : Glcd_wYpos_S holds the starting Y position
'           : Glcd_wLength holds the length of the rectangle
'           : Glcd_wHeight holds the height of the rectangle
' Output    : None
' Notes     :
'
$define Glcd_Rectangle(pXpos, pYpos, pLength, pHeight) '
    Glcd_wYpos_S = pYpos '
    Glcd_wLength = pLength '
    Glcd_wHeight = pHeight '
    WREG = pXpos '
    _Glcd_Rectangle

_Glcd_Rectangle Macro-
    GoSub __Glcd_Rectangle
Endm

#ifMacro- _Glcd_Rectangle                   ' Add this block only if the macro has been used in the program
__Glcd_Rectangle:
    Glcd_wXpos_S = WREG                     ' Transfer WREG into its working variable
    Glcd_Line(Glcd_wXpos_S, Glcd_wYpos_S, Glcd_wXpos_S + Glcd_wLength, Glcd_wYpos_S)  ' Draw top line

    Glcd_wYpos_S = Glcd_wYpos_E             ' Transfer previous Y pos end to Y pos start
    Glcd_wXpos_S = Glcd_bXpos_E             ' Transfer previous X pos end to X pos start
    Glcd_wYpos_E = Glcd_wYpos_E + Glcd_wHeight
    GoSub __Glcd_LineP                      ' Draw right line

    Glcd_wYpos_S = Glcd_wYpos_E             ' Transfer previous Y pos end to Y pos start
    Glcd_wXpos_S = Glcd_bXpos_E             ' Transfer previous X pos end to X pos start
    Glcd_wXpos_E = Glcd_wXpos_E - Glcd_bLength
    GoSub __Glcd_LineP                      ' Draw bottom line

    Glcd_wYpos_S = Glcd_wYpos_E             ' Transfer previous Y pos end to Y pos start
    Glcd_wXpos_S = Glcd_bXpos_E             ' Transfer previous X pos end to X pos start
    Glcd_wYpos_E = Glcd_wYpos_E - Glcd_wHeight
    GoSub __Glcd_LineP                      ' Draw left line
    Return
#endIfMacro-
'-------------------------------------------------------------------------------------------------
' Move the cursor values (in pixels)
' Input     : Glcd_wCursorY = 16-bit Y position
'           : WREG = 8-bit X position
' Output    : None
' Notes     : None
'
$define Glcd_Cursor(pXpos, pYpos) '
    Glcd_wCursorY = pYpos '
    Glcd_bCursorX = pXpos

'-------------------------------------------------------------------------------------------------
' Clear the LCD's screen with the current colour
' Input     : ILI_wProd = screen colour
' Output    : None
' Notes     : None
'
$define Glcd_Cls(pColour) '
    ILI_wProd = pColour '
    _Glcd_Cls

_Glcd_Cls Macro-
    GoSub __Glcd_Cls
Endm

#ifMacro- _Glcd_Cls
__Glcd_Cls:
    ILI_wTblPtr = wPixelColour      ' Save the pixel's colour
    wPixelColour = ILI_wProd
    hSolidFill(__LCD_X_Res * __LCD_Y_Res)
    Glcd_Cursor(0,0)
    wPixelColour = ILI_wTblPtr      ' Restore the pixel's colour
    Return
#endIfMacro-
'-------------------------------------------------------------------------------------------------
' Choose a font for character display
' Input     : TBLPTRL\H hold the font offset table address
' Output    : Glcd_wHeaderAddress holds the start of the font offset table address
'           : Glcd_bFontHeight holds the height of the font (in pixels)
'           : Glcd_bFontAmountReads holds the amount of bytes per font segment
' Notes     : None
'
$define Glcd_SetFont(pFont) '
    ILI_wTblPtr = VarPtr pFont '
    _SetFont

_SetFont Macro-
    GoSub __SetFont
    Endm

#ifMacro- _SetFont
__SetFont:
    Tblrd*+                                             ' \ Read and store the font height from the font table
    Glcd_bFontHeight = TABLAT                           ' /
    Tblrd*+                                             ' Dummy read to increment the code memory address
    Glcd_wHeaderAddress = ILI_wTblPtr                   ' Store the current location of the font offset table
'
' Find the amount of bytes per font segment
'
    Glcd_bFontAmountReads = Glcd_bFontHeight / 8        ' Find the amount of bytes
    PRODL = Glcd_bFontHeight // 8                       ' Find the remainder
    If PRODL <> 0 Then                                  ' Is there a remainder?
        Inc Glcd_bFontAmountReads                       ' Yes. So increment the amount of bytes
    EndIf
    Return
#endIfMacro-
'-------------------------------------------------------------------------------------------------
' Display an ASCII character
' Input     : WREG holds the character to display
'           : If tTransparent = true then do not use a paper colour
'           : If tVerticalPrint = true then the characters will be displayed vertically
' Output    : WREG still contains the character that was displayed
' Notes     : Glcd_wCursorY holds the Y position (in pixels)
'           : Glcd_bCursorX holds the X position (in pixels)
'
$define Glcd_PrintChar(pChar) WREG = pChar : _Glcd_PrintChar

_Glcd_PrintChar Macro-
    GoSub __Glcd_PrintChar
    Endm

#ifdef Print#Req
@Print
__Glcd_PrintChar:
    Glcd_wSaveTblPtr = ILI_wTblPtr                         ' Save the contents of TBLPTRL\H
'
' Locate the character's image data from code memory tables
'
    Glcd_bSaveChar = WREG                                   ' Save the character value
    WREG = WREG - 32                                        ' Remove the ASCII offset
    WREG = WREG * 2                                         ' Offsets are 2 bytes each (words)
    ILI_wTblPtr = Glcd_wHeaderAddress + WREG                ' Point to the font image offset within the the table
    Tblrd*+                                                 ' \ Read and store the low byte of the image address
    ILI_wProd.Byte0 = TABLAT                                ' /
    Tblrd*                                                  ' \ Read and store the high byte of the image address
    ILI_wProd.Byte1 = TABLAT                                ' /
    ILI_wTblPtr = ILI_wProd                                 ' Point to the image address of the font
    Tblrd*+                                                 ' \ Read the font width value then increment to skip over it
    Glcd_bFontWidth = TABLAT                                ' /
'
' Place the character on the LCD
'
    If tVerticalPrint = False Then                          ' Do we require vertical printing?
        Glcd_WriteReg($03,%0001000000111000)                ' No. So set scanning mode for Horizontal writes, Horizontal inrement, Vertical increment
        Glcd_SetWindow(Glcd_bCursorX, Glcd_wCursorY, (Glcd_bCursorX + Glcd_bFontWidth) - 1, (Glcd_wCursorY + Glcd_bFontHeight) - 1) ' Create a window for the size of the character
        Clear Glcd_CS                                       ' Enable the LCD
        hSetCGRAM(Glcd_bCursorX, Glcd_wCursorY)             ' Point to CGRAM
        Glcd_bCursorX = Glcd_bCursorX + Glcd_bFontWidth     ' Update the cursor position ready for the next character
    Else                                                    ' Otherwise... Use vertical print
        Glcd_WriteReg($03,%0001000000010000)                ' Set scanning mode For Horizontal writes, horizontal increment, Vertical decrement
        Glcd_SetWindow(Glcd_bCursorX, Glcd_wCursorY, ((Glcd_bCursorX + Glcd_bFontHeight) - 1), ((Glcd_wCursorY + Glcd_bFontWidth) - 1)) ' Create a window for the size of the character
        Clear Glcd_CS                                       ' Enable the LCD
        hSetCGRAM(Glcd_bCursorX, ((Glcd_wCursorY + Glcd_bFontWidth) - 1)))  ' Point to CGRAM
        Glcd_wCursorY = Glcd_wCursorY - Glcd_bFontWidth     ' Move the cursor position for the next character
    EndIf
_FontWidthLoop:
        Glcd_bHeightCount = Glcd_bFontHeight                ' Set the character height counter
        Glcd_bCharReadsTemp = Glcd_bFontAmountReads         ' Store the amount of bytes to read for this segment
_FontBytesLoop:
        Tblrd*+                                             ' Read the byte from code memory
        Glcd_bCharPixelsTemp = 8                            ' Create a loop for the bits to plot
_CharByteLoop:
            If tTransparent = False Then                    ' Do we require transparent background?
                ILI_wProd = wPixelColour                    ' No. So save the colour of the pixel
                If TABLAT.0 = 0 Then                        ' Do we need to set a pixel?
                    wPixelColour = wBackGroundColour        ' No. So use the paper (background) colour
                EndIf
                hWriteDataWord(wPixelColour)                ' Place a pixel on the LCD
                wPixelColour = ILI_wProd                    ' Restore the colour of the pixel
            Else                                            ' Otherwise...
                If TABLAT.0 = 1 Then                        ' Do we need to set a pixel?
                    hWriteDataWord(wPixelColour)            ' Yes. So place a pixel on the LCD
                EndIf
            EndIf
            Ror TABLAT                                      ' Move each pixel into bit-0
            Dec Glcd_bHeightCount                           ' Decremrnt the character font height counter
            If Glcd_bHeightCount = 0 Then ExitLoop          ' Exit the loop when we reach the correct size of the character
            Djnz Glcd_bCharPixelsTemp,_CharByteLoop         ' Close the loop when all pixels plotted
        Djnz Glcd_bCharReadsTemp,_FontBytesLoop             ' Close the loop when all bytes read
ExitLoop:
    Djnz Glcd_bFontWidth,_FontWidthLoop                    ' Close the loop when all width pixels are done
    Set Glcd_CS
    Glcd_WriteReg($03,%0001000000110000)                    ' Set Entry mode for horizontal left to right increments
    ILI_wTblPtr = Glcd_wSaveTblPtr                          ' Restore the contents of TBLPTRL\H
    WREG = Glcd_bSaveChar                                   ' Restore the original character in WREG before returning
    Return
#endif

'-------------------------------------------------------------------------------------------------
' Display text
' Input     : Uses a single modifier from the compiler's Print routine
' Output    : None
' Notes     : This routine hi-jacks the compiler's Print library
'
$define Glcd_Print(pStringName) _Glcd_Print : Print pStringName
'
' Create a dummy macro to trigger the inclusion of the font table
'
_Glcd_Print Macro pStringName
Endm
'-------------------------------------------------------------------------------------------------
' Helper routine to scroll the display
' Input     : Glcd_wParam holds tha amount of scrolls (0 to 319)
' Output    : None
' Notes     : NOne
'
$define _hScroll(pAmount) '
    Glcd_wParam = pAmount '
    _hScroll_

_hScroll_ Macro-
    GoSub __hScroll
    Endm

#ifMacro- _hScroll_,_Glcd_ScrollDown, _Glcd_ScrollUp
__hScroll:
    While Glcd_wParam.15 = 1
        Glcd_wParam = Glcd_wParam + 320
    Wend
    While Glcd_wParam >= 320
        Glcd_wParam = Glcd_wParam - 320
    Wend
    Glcd_WriteReg($6A,Glcd_wParam)      ' Do the scroll
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Scroll the display Down
' Input     : Glcd_wParam holds tha amount of scrolls (0 to 319)
' Output    : None
' Notes     : NOne
'
$define Glcd_ScrollDown(pAmount) '
    Glcd_wParam = pAmount    '
    _Glcd_ScrollDown

_Glcd_ScrollDown Macro-
    GoSub __Glcd_ScrollDown
    Endm

#ifMacro- _Glcd_ScrollDown
__Glcd_ScrollDown:
    _hScroll(-Glcd_wParam)      ' Do the scroll
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Scroll the display Up
' Input     : Glcd_wParam holds tha amount of scrolls (0 to 319)
' Output    : None
' Notes     : NOne
'
$define Glcd_ScrollUp(pAmount) '
    Glcd_wParam = pAmount    '
    _Glcd_ScrollUp

_Glcd_ScrollUp Macro-
    GoSub __Glcd_ScrollUp
    Endm

#ifMacro- _Glcd_ScrollUp
__Glcd_ScrollUp:
    _hScroll(Glcd_wParam)      ' Do the scroll
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Draw a solid rectangle
' Input     : Glcd_bX1 holds the X position start (0 to 239)
'           : Glcd_bY1 holds the Y position start (0 to 319)
'           : Glcd_bX2 holds the X position end (0 to 239)
'           : Glcd_bY2 holds the Y position end (0 to 319)
' Output    : None
' Notes     :
'
$define Glcd_SolidRectangle(pXstart, pYstart, pXend, pYend) '
    Glcd_bX2 = pXend   '
    Glcd_bY1 = pYstart '
    Glcd_bY2 = pYend   '
    WREG = pXstart     '
    _Glcd_SolidRectangle

_Glcd_SolidRectangle Macro-
    GoSub __Glcd_SolidRectangle
Endm

#ifMacro- _Glcd_SolidRectangle
__Glcd_SolidRectangle:
    Glcd_bX1 = WREG
    Glcd_SetWindow(Glcd_bX1, Glcd_bY1, Glcd_bX2, Glcd_bY2)  ' Create a window on the screen of the size we require for the rectangle
    Clear Glcd_CS                                   ' Re-Enable the LCD
    hSetCGRAM(Glcd_bX1,Glcd_bY1)                    ' Move to the correct location
    Glcd_bX2 = Glcd_bX2 - Glcd_bX1
    Inc Glcd_bX2
    Glcd_bY2 = Glcd_bY2 - Glcd_bY1
    Inc Glcd_bY2
    hSolidFill(Glcd_bX2 * Glcd_bY2)
    Glcd_ResetWindow()
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Place an image on the LCD from code memory
' Input     : ILI_wTblPtr holds the address of the image in code memory
'           : Glcd_bPixelXpos holds the X position of the image (0 to 239)
'           : Glcd_wPixelYpos holds the Y position of the image (0 to 319)
' Output    : None
' Notes     : Reads the amount of bytes in the image and its length and width from code memory
'
$define Glcd_ImageFromCode(pXpos, pYpos, pAddress) '
    Glcd_wPixelYpos = pYpos '
    ILI_wTblPtr = VarPtr pAddress '
    WREG = pXpos '
    _Glcd_ImageFromCode

_Glcd_ImageFromCode Macro-
    GoSub __Glcd_ImageFromCode
Endm

#ifMacro- _Glcd_ImageFromCode
__Glcd_ImageFromCode:
    Glcd_bPixelXpos = WREG          ' Transfer the WREG into its working variable
'
' Read the header from the image data
'
    Tblrd*+                         ' Read the low byte of the image size data item
    Glcd_wImageSize.Byte0 = TABLAT
    Tblrd*+                         ' Read the high byte of the image size data item
    Glcd_wImageSize.Byte1 = TABLAT
    Tblrd*+                         ' Read the low byte of the width data item
    Glcd_bWidth = TABLAT
    Tblrd*+                         ' Dummy read of the high byte of the width data item
    Tblrd*+                         ' Read the low byte of the height data item
    Glcd_wHeight.Byte0 = TABLAT
    Tblrd*+                         ' Read the high byte of the height data item
    Glcd_wHeight.Byte1 = TABLAT

    Glcd_SetWindow(Glcd_bPixelXpos, Glcd_wPixelYpos, (Glcd_bWidth + Glcd_bPixelXpos), (Glcd_wHeight + Glcd_wPixelYpos)) ' Create a window for the image to be placed into
    Clear Glcd_CS                   ' Enable the LCD
    hSetCGRAM(Glcd_bPixelXpos,Glcd_wPixelYpos)' Move to the correct location
'
' Draw the image on the LCD
'
    Repeat
        Tblrd*+                     ' Read the low byte of the pixel colour data item
        Glcd_wParam.Byte0 = TABLAT
        Tblrd*+                     ' Read the high byte of the pixel colour data item
        Glcd_wParam.Byte1 = TABLAT
        GoSub __hWriteData_P        ' Vector for hWriteData to call with Glcd_wParam loaded with the pixel colour
        Dec Glcd_wImageSize
    Until Glcd_wImageSize = 0
    Glcd_ResetWindow()              ' Reset the window to full size before we leave
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Place an image on the LCD from code memory with magnification
' Input     : ILI_wTblPtr holds the address of the image in code memory
'           : Glcd_bPixelXpos holds the X position of the image (0 to 239)
'           : Glcd_wPixelYpos holds the Y position of the image (0 to 319)
'           : Glcd_bMag holds the magnification factor of the image
' Output    : None
' Notes     : Reads the amount of bytes in the image and its length and width from code memory
'           : Will not display correctly if the image does not fit on the screen
'
$define Glcd_ImageFromCodeMag(pXpos, pYpos, pAddress, pMag) '
    Glcd_wPixelYpos = pYpos '
    ILI_wTblPtr = VarPtr pAddress '
    Glcd_bMag = pMag '
    WREG = pXpos '
    _Glcd_ImageFromCodeMag

_Glcd_ImageFromCodeMag Macro-
    GoSub __Glcd_ImageFromCodeMag
Endm

#ifMacro- _Glcd_ImageFromCodeMag

__Glcd_ImageFromCodeMag:
    Glcd_bPixelXpos = WREG          ' Transfer the WREG into its working variable
'
' Read the header from the image data
'
    Tblrd*+                         ' Read the low byte of the image size data item
    Glcd_wImageSize.Byte0 = TABLAT
    Tblrd*+                         ' Read the high byte of the image size data item
    Glcd_wImageSize.Byte1 = TABLAT
    Tblrd*+                         ' Read the low byte of the width data item
    Glcd_bWidth = TABLAT
    Tblrd*+                         ' Dummy read of the high byte of the width data item
    Tblrd*+                         ' Read the low byte of the height data item
    Glcd_wHeight.Byte0 = TABLAT
    Tblrd*+                         ' Read the high byte of the height data item
    Glcd_wHeight.Byte1 = TABLAT
'
' Calculate the window size required
'
    Glcd_bMultMinusOne = Glcd_bMag - 1
    Glcd_bWinEndXPos = Glcd_bWidth * Glcd_bMag
    Glcd_bWinEndXPos = Glcd_bWinEndXPos + Glcd_bPixelXpos
    Glcd_bWinEndXPos = Glcd_bWinEndXPos + Glcd_bMultMinusOne
    Glcd_wWinEndYPos = Glcd_wHeight * Glcd_bMag
    Glcd_wWinEndYPos = Glcd_wWinEndYPos + Glcd_wPixelYpos
    Glcd_wWinEndYPos = Glcd_wWinEndYPos + Glcd_bMultMinusOne
'
' Check and correct boundaries
'
    If Glcd_bWinEndXPos >= 239 Then Glcd_bWinEndXPos = 239
    If Glcd_wWinEndYPos >= 319 Then Glcd_wWinEndYPos = 319
    Glcd_SetWindow(Glcd_bPixelXpos, Glcd_wPixelYpos, Glcd_bWinEndXPos, Glcd_wWinEndYPos) ' Create a window for the image to be placed into

    Clear Glcd_CS                                   ' Enable the LCD
    hSetCGRAM(Glcd_bPixelXpos,Glcd_wPixelYpos)      ' Move to the correct location
'
' Draw the image on the LCD
'
    Repeat
        Glcd_wStartOfLine = ILI_wTblPtr             ' Save the position of the start of a line in code memory
        Glcd_bDown = Glcd_bMag
        Repeat                                      ' Create a loop for the amount of pixels down
            ILI_wTblPtr = Glcd_wStartOfLine         ' Restore the position of the start of the line
            For Glcd_bAcross = Glcd_bWidth To 0 Step -1  ' Scan across the line
                Tblrd*+                             ' Read the low byte of the pixel colour data item
                Glcd_wParam.Byte0 = TABLAT
                Tblrd*+                             ' Read the high byte of the pixel colour data item
                Glcd_wParam.Byte1 = TABLAT
                Glcd_bPixelWidth = Glcd_bMag
                Repeat
                    GoSub __hWriteData_P            ' Vector for hWriteData to call with Glcd_wParam loaded with the pixel colour
                    Dec Glcd_bPixelWidth
                Until Glcd_bPixelWidth = 0
                If Glcd_bDown = 1 Then Dec Glcd_wImageSize    ' Decrement only once per loop
            Next
            Dec Glcd_bDown
        Until Glcd_bDown = 0
    Until Glcd_wImageSize = 0
    Glcd_ResetWindow()                              ' Reset the window to full size before we leave
    Return
#endIfMacro-
'-------------------------------------------------------------------------------------------------
' Place an 8-bit RGB332 image on the LCD from code memory
' Input     : ILI_wTblPtr holds the address of the image in code memory
'           : Glcd_bPixelXpos holds the X position of the image (0 to 239)
'           : Glcd_wPixelYpos holds the Y position of the image (0 to 319)
' Output    : None
' Notes     : Reads the amount of bytes in the image and its length and width from code memory
'           : RGB332 Format is RRRGGGBB
'
$define Glcd_Image8_FromCode(pXpos, pYpos, pAddress) '
    Glcd_wPixelYpos = pYpos '
    ILI_wTblPtr = VarPtr pAddress '
    WREG = pXpos '
    _Glcd_Image8_FromCode

_Glcd_Image8_FromCode Macro-
    GoSub __Glcd_Image8_FromCode
Endm

#ifMacro- _Glcd_Image8_FromCode
__Glcd_Image8_FromCode:
    Glcd_bPixelXpos = WREG          ' Transfer the WREG into its working variable
'
' Read the header from the image data
'
    Tblrd*+                         ' Read the low byte of the image size data item
    Glcd_wImageSize.Byte0 = TABLAT
    Tblrd*+                         ' Read the high byte of the image size data item
    Glcd_wImageSize.Byte1 = TABLAT
    Tblrd*+                         ' Read the low byte of the width data item
    Glcd_bWidth = TABLAT
    Tblrd*+                         ' Dummy read of the high byte of the width data item
    Tblrd*+                         ' Read the low byte of the height data item
    Glcd_wHeight.Byte0 = TABLAT
    Tblrd*+                         ' Read the high byte of the height data item
    Glcd_wHeight.Byte1 = TABLAT

    Glcd_SetWindow(Glcd_bPixelXpos, Glcd_wPixelYpos, (Glcd_bWidth + Glcd_bPixelXpos), (Glcd_wHeight + Glcd_wPixelYpos)) ' Create a window for the image to be placed into
    Clear Glcd_CS                   ' Enable the LCD
    hSetCGRAM(Glcd_bPixelXpos,Glcd_wPixelYpos)' Move to the correct location
'
' Draw the image on the LCD
'
    Repeat
        Tblrd*+                     ' Read the byte of the RGB332 pixel colour data item into TABLAT
        If TABLAT = 0 Then          ' Is the pixel colour 0?
            Glcd_wParam = $00       ' Yes. So no need for a conversion
        ElseIf TABLAT = 255 Then    ' Is the pixel colour 255?
            Glcd_wParam = $FFFF     ' Yes. So no need for a conversion
        Else                        ' Otherwise...
            '
            ' Convert the 8-bit RGB332 value into a 16-bit RGB565 value
            '
            Glcd_wParam = %0001100011100111  ' Create a mask
            '
            ' Place the blue content
            '
            WREG = TABLAT & %00000011
            WREG = WREG << 3
            Glcd_wParam.Byte0 = Glcd_wParam.Byte0 | WREG
            '
            ' Place the red content
            '
            WREG = TABLAT & %11100000
            Glcd_wParam.Byte1 = Glcd_wParam.Byte1 | WREG
            '
            ' Place the green content
            '
            WREG = TABLAT & %00011100
            WREG = WREG >> 2
            Glcd_wParam.Byte1 = Glcd_wParam.Byte1 | WREG
        EndIf
        GoSub __hWriteData_P        ' Vector for hWriteData to call with Glcd_wParam loaded with the pixel colour
        Dec Glcd_wImageSize
    Until Glcd_wImageSize = 0
    Glcd_ResetWindow()              ' Reset the window to full size before we leave
    Return
#endIfMacro-
'-------------------------------------------------------------------------------------------------
' Place an image on the LCD from code memory with magnification
' Input     : ILI_wTblPtr holds the address of the image in code memory
'           : Glcd_bPixelXpos holds the X position of the image (0 to 239)
'           : Glcd_wPixelYpos holds the Y position of the image (0 to 319)
'           : Glcd_bMag holds the magnification factor of the image
' Output    : None
' Notes     : Reads the amount of bytes in the image and its length and width from code memory
'           : Will not display correctly if the image does not fit on the screen
'
$define Glcd_Image8_FromCodeMag(pXpos, pYpos, pAddress, pMag) '
    Glcd_wPixelYpos = pYpos '
    ILI_wTblPtr = VarPtr pAddress '
    Glcd_bMag = pMag '
    WREG = pXpos '
    _Glcd_Image8_FromCodeMag

_Glcd_Image8_FromCodeMag Macro-
    GoSub __Glcd_Image8_FromCodeMag
Endm

#ifMacro- _Glcd_Image8_FromCodeMag

__Glcd_Image8_FromCodeMag:
    Glcd_bPixelXpos = WREG          ' Transfer the WREG into its working variable
'
' Read the header from the image data
'
    Tblrd*+                         ' Read the low byte of the image size data item
    Glcd_wImageSize.Byte0 = TABLAT
    Tblrd*+                         ' Read the high byte of the image size data item
    Glcd_wImageSize.Byte1 = TABLAT
    Tblrd*+                         ' Read the low byte of the width data item
    Glcd_bWidth = TABLAT
    Tblrd*+                         ' Dummy read of the high byte of the width data item
    Tblrd*+                         ' Read the low byte of the height data item
    Glcd_wHeight.Byte0 = TABLAT
    Tblrd*+                         ' Read the high byte of the height data item
    Glcd_wHeight.Byte1 = TABLAT
'
' Calculate the window size required
'
    Glcd_bMultMinusOne = Glcd_bMag - 1
    Glcd_bWinEndXPos = Glcd_bWidth * Glcd_bMag
    Glcd_bWinEndXPos = Glcd_bWinEndXPos + Glcd_bPixelXpos
    Glcd_bWinEndXPos = Glcd_bWinEndXPos + Glcd_bMultMinusOne
    Glcd_wWinEndYPos = Glcd_wHeight * Glcd_bMag
    Glcd_wWinEndYPos = Glcd_wWinEndYPos + Glcd_wPixelYpos
    Glcd_wWinEndYPos = Glcd_wWinEndYPos + Glcd_bMultMinusOne
'
' Check and correct boundaries
'
    If Glcd_bWinEndXPos >= 239 Then Glcd_bWinEndXPos = 239
    If Glcd_wWinEndYPos >= 319 Then Glcd_wWinEndYPos = 319
    Glcd_SetWindow(Glcd_bPixelXpos, Glcd_wPixelYpos, Glcd_bWinEndXPos, Glcd_wWinEndYPos) ' Create a window for the image to be placed into
    Clear Glcd_CS                                   ' Enable the LCD
    hSetCGRAM(Glcd_bPixelXpos,Glcd_wPixelYpos)      ' Move to the correct location
'
' Draw the image on the LCD
'
    Repeat
        Glcd_wStartOfLine = ILI_wTblPtr             ' Save the position of the start of a line in code memory
        Glcd_bDown = Glcd_bMag
        Repeat                                      ' Create a loop for the amount of pixels down
            ILI_wTblPtr = Glcd_wStartOfLine         ' Restore the position of the start of the line
            For Glcd_bAcross = Glcd_bWidth To 0 Step -1  ' Scan across the line
                Tblrd*+                             ' Read the byte of the RGB332 pixel colour data item into TABLAT
                If TABLAT = 0 Then                  ' Is the pixel colour 0?
                    Glcd_wParam = $00               ' Yes. So no need for a conversion
                ElseIf TABLAT = 255 Then            ' Is the pixel colour 255?
                    Glcd_wParam = $FFFF             ' Yes. So no need for a conversion
                Else                                ' Otherwise...
                    '
                    ' Convert the 8-bit RGB332 value into a 16-bit RGB565 value
                    '
                    Glcd_wParam = %0001100011100111  ' Create a mask
                    '
                    ' Place the blue content
                    '
                    WREG = TABLAT & %00000011
                    WREG = WREG << 3
                    Glcd_wParam.Byte0 = Glcd_wParam.Byte0 | WREG
                    '
                    ' Place the red content
                    '
                    WREG = TABLAT & %11100000
                    Glcd_wParam.Byte1 = Glcd_wParam.Byte1 | WREG
                    '
                    ' Place the green content
                    '
                    WREG = TABLAT & %00011100
                    WREG = WREG >> 2
                    Glcd_wParam.Byte1 = Glcd_wParam.Byte1 | WREG
                EndIf
                Glcd_bPixelWidth = Glcd_bMag
                Repeat
                    GoSub __hWriteData_P            ' Vector for hWriteData to call with Glcd_wParam loaded with the pixel colour
                    Dec Glcd_bPixelWidth
                Until Glcd_bPixelWidth = 0
                If Glcd_bDown = 1 Then Dec Glcd_wImageSize    ' Decrement only once per loop
            Next
            Dec Glcd_bDown
        Until Glcd_bDown = 0
    Until Glcd_wImageSize = 0
    Glcd_ResetWindow()                              ' Reset the window to full size before we leave
    Return
#endIfMacro-
'-------------------------------------------------------------------------------------------------
_ILI9320_Main_:
    DelayMS Glcd_InitDelay
    Glcd_Init()

$endif
