$ifndef _Map_
$define _Map_
(*

    Map(Value, FromLow, FromHigh, ToLow, ToHigh, Result)

Scales a number from one range to another. That is, a value of fromLow would get Mapped to toLow, a value of fromHigh to toHigh, values in-between to values in-between, etc.
Does not constrain values to within the range, because out-of-range values are sometimes intended and useful.
Note that the "lower bounds" of either range MUST be smaller than the "upper bounds"
The macro does NOT handle negative numbers without changes made to it

The Map() macro uses integer math so will not generate fractions. Fractional remainders are truncated.

Parameters:

    Value is the value to scale

    FromLow is the lower bound of the value's current range

    FromHigh is the upper bound of the value's current range

    ToLow is the lower bound of the value's target range

    ToHigh is the upper bound of the value's target range

    Result is the result of the calculation

Example:

' Scale a 10-bit (0 to 1023) value to 8-bits (0 to 255)

    Dim Value as Word
    Dim Result as Byte

    For Value = 0 to 1023
        Map(Value, 0, 1023, 0, 255, Result);
        Hrsout Dec Value, " = ", Dec Result, 13
    Next

Note:
The calculation used by Map is:

    Result =  ((x_in - in_min) * (out_max - out_min) / (in_max - in_min)) + out_min
*)
'
' Declare compiler system variables used within the Map subroutine
'
    Dim PP0 As Byte System
    Dim PP0H As Byte System
    Dim PP7 As Byte System
    Dim PP7H As Byte System
'
' Declare variables used as parameters and used within the Scale subroutine
' 
#ifMacro- _Map8
    Dim Scl_bTemp1 As PP7
    Dim Scl_bTemp2 As PP7H
    Dim Scl_wTemp As PP0.Word           ' Result is the result of the calculation
    
    Dim Scl_bXin As Scl_bTemp1          ' Value is the value to Scale
    Dim Scl_bInMin As Byte              ' Lower bound of the value's current range
    Dim Scl_bInMax As Byte              ' Upper bound of the value's current range
    Dim Scl_bOutMin As Byte             ' Lower bound of the value's target range
    Dim Scl_bOutMax As Scl_bTemp2       ' Upper bound of the value's target range
#endIfMacro-

#ifMacro- _Map16
    Dim PP0HH As Byte System
    Dim PP0HHH As Byte System
    Dim PP7HH As Byte System
    Dim PP7HHH As Byte System
    Dim Scl_wTemp1 As PP7.Word
    Dim Scl_wTemp2 As PP7HH.Word
    Dim Scl_dTemp As PP0.Dword          ' Result is the result of the calculation
    
    Dim Scl_wXin As Scl_wTemp1          ' Value is the value to Scale
    Dim Scl_wInMin As Word              ' Lower bound of the value's current range
    Dim Scl_wInMax As Word              ' Upper bound of the value's current range
    Dim Scl_wOutMin As Word             ' Lower bound of the value's target range
    Dim Scl_wOutMax As Scl_wTemp2       ' Upper bound of the value's target range
#endIfMacro-

#ifMacro- _Map32
    Dim PP0HH As Byte System
    Dim PP0HHH As Byte System
    Dim PP8 As Byte System
    Dim PP8H As Byte System
    Dim PP7HH As Byte System
    Dim PP7HHH As Byte System
    Dim PP8HH As Byte System
    Dim PP8HHH As Byte System
    Dim Scl_dTemp1 As PP7.Dword
    Dim Scl_dTemp2 As PP8.Dword
    Dim Scl_dTemp As PP0.Dword          ' Result is the result of the calculation
    
    Dim Scl_dXin As Scl_dTemp1          ' Value is the value to Scale
    Dim Scl_dInMin As Dword             ' Lower bound of the value's current range
    Dim Scl_dInMax As Dword             ' Upper bound of the value's current range
    Dim Scl_dOutMin As Dword            ' Lower bound of the value's target range
    Dim Scl_dOutMax As Scl_dTemp2       ' Upper bound of the value's target range
#endIfMacro-

#ifMacro- _MapF   
    Dim PP0HH As Byte System
    Dim PP0HHH As Byte System
    Dim PP8 As Byte System
    Dim PP8H As Byte System
    Dim PP7HH As Byte System
    Dim PP7HHH As Byte System
    Dim PP8HH As Byte System
    Dim PP8HHH As Byte System
    Dim Scl_fTemp1 As PP7.Float
    Dim Scl_fTemp2 As PP8.Float
    Dim Scl_fTemp As PP0.Float          ' Result is the result of the calculation
    
    Dim Scl_fXin As Scl_fTemp1          ' Value is the value to Scale
    Dim Scl_fInMin As Float             ' Lower bound of the value's current range
    Dim Scl_fInMax As Float             ' Upper bound of the value's current range
    Dim Scl_fOutMin As Float            ' Lower bound of the value's target range
    Dim Scl_fOutMax As Scl_fTemp2       ' Upper bound of the value's target range
#endIfMacro-
'-----------------------------------------------------------------------------------------
    GoTo _ScaleMain
'-----------------------------------------------------------------------------------------   
' 8-bit scale
' Input     :
' Output    : pResult holds the result
' Notes     : None
'
$define Map8(pX, pInMin, pInMax, pOutMin, pOutMax, pResult) ' 
    Scl_bXin = pX         '
    Scl_bInMin = pInMin   '
    Scl_bInMax = pInMax   '
    Scl_bOutMin = pOutMin '
    WREG = pOutMax        '
    _Map8                 '
    pResult = Scl_wTemp

_Map8 Macro-
    GoSub __Map8
    Endm  

#ifMacro- _Map8   
__Map8:    
    Scl_bOutMax = WREG
    Scl_bTemp1 = Scl_bXin - Scl_bInMin
    Scl_bTemp2 = Scl_bOutMax - Scl_bOutMin   
    Scl_wTemp = Scl_bTemp1 * Scl_bTemp2
    Scl_bTemp1 = Scl_bInMax - Scl_bInMin    
    Scl_bTemp2 = Scl_wTemp / Scl_bTemp1  
    Scl_wTemp = Scl_bTemp2 + Scl_bOutMin
    Return    
#endIfMacro-
'-----------------------------------------------------------------------------------------
' 16-bit scale
' Input     :
' Output    : pResult holds the result
' Notes     : None
'
$define Map16(pX, pInMin, pInMax, pOutMin, pOutMax, pResult) '
    Scl_wXin = pX         '
    Scl_wInMin = pInMin   '
    Scl_wInMax = pInMax   '
    Scl_wOutMin = pOutMin '
    Scl_wOutMax = pOutMax '
    _Map16                '
    pResult = Scl_dTemp

_Map16 Macro-
    GoSub __Map16
    Endm

#ifMacro- _Map16
__Map16:
    Scl_wTemp1 = (Scl_wXin - Scl_wInMin)
    Scl_wTemp2 = (Scl_wOutMax - Scl_wOutMin)
    Scl_dTemp = Scl_wTemp1 * Scl_wTemp2
    Scl_wTemp1 = (Scl_wInMax - Scl_wInMin)
    Scl_wTemp2 = Scl_dTemp / Scl_wTemp1
    Scl_dTemp = Scl_wTemp2 + Scl_wOutMin
    Return
#endIfMacro-
'-----------------------------------------------------------------------------------------   
' 32-bit scale
' Input     :
' Output    : pResult holds the result
' Notes     : None
'
$define Map32(pX, pInMin, pInMax, pOutMin, pOutMax, pResult) ' 
    Scl_dXin = pX         '
    Scl_dInMin = pInMin   '
    Scl_dInMax = pInMax   '
    Scl_dOutMin = pOutMin '
    Scl_dOutMax = pOutMax '
    _Map32                '
    pResult = Scl_dTemp

_Map32 Macro-
    GoSub __Map32
    Endm  

#ifMacro- _Map32   
__Map32:    
    Scl_dTemp1 = Scl_dXin - Scl_dInMin
    Scl_dTemp2 = Scl_dOutMax - Scl_dOutMin   
    Scl_dTemp = Scl_dTemp1 * Scl_dTemp2
    Scl_dTemp1 = Scl_dInMax - Scl_dInMin    
    Scl_dTemp2 = Scl_dTemp / Scl_dTemp1  
    Scl_dTemp = Scl_dTemp2 + Scl_dOutMin
    Return    
#endIfMacro-

'-----------------------------------------------------------------------------------------   
' Floating point scale
' Input     :
' Output    : pResult holds the floating point result
' Notes     : None
'
$define MapF(pX, pInMin, pInMax, pOutMin, pOutMax, pResult) ' 
    Scl_fXin = pX         '
    Scl_fInMin = pInMin   '
    Scl_fInMax = pInMax   '
    Scl_fOutMin = pOutMin '
    Scl_fOutMax = pOutMax '
    _MapF                 '
    pResult = Scl_fTemp

_MapF Macro-
    GoSub __MapF
    Endm  

#ifMacro- _MapF   
__MapF:    
    Scl_fTemp1 = Scl_fXin - Scl_fInMin
    Scl_fTemp2 = Scl_fOutMax - Scl_fOutMin   
    Scl_fTemp = Scl_fTemp1 * Scl_fTemp2
    Scl_fTemp1 = Scl_fInMax - Scl_fInMin    
    Scl_fTemp2 = Scl_fTemp / Scl_fTemp1  
    Scl_fTemp = Scl_fTemp2 + Scl_fOutMin
    Return    
#endIfMacro-
'-----------------------------------------------------------------------------------------
' Make sure a value remains within boundaries
'
$define Constrain(pValue, pMin, pMax, pResult) '
    If pValue < pMin Then     '
        pResult = pMin        '
    ElseIf pValue > pMax Then '
        pResult = pMax        '
    EndIf

'-----------------------------------------------------------------------------------------
_ScaleMain:

$endif

