$ifndef _ADS7846_
$define _ADS7846_
'
' Interface to an ADS7846 Touch Screen Controller
' Written by Les Johnson. September 2011
'
    Include "SPI.inc"                               ' Load the SPI macros into the program
'
' ADS7846 SPI interface pins
' Default setting for use with the Amicus Touch Screen Colour LCD Shield
' 
$ifndef ADS_SDO   
    $define ADS_SDO PORTC.5                         ' SPI SDO to ADS7846 DIN (Uses the MSSP1 peripheral)
$endif
$ifndef ADS_SDI
    $define ADS_SDI PORTC.4                         ' SPI SDI from ADS7846 DOUT (Uses the MSSP1 peripheral)
$endif
$ifndef ADS_SCK
    $define ADS_SCK PORTC.3                         ' SPI SCK to ADS7846 CLK  (Uses the MSSP1 peripheral)
$endif
$ifndef ADS_CS
    $define ADS_CS PORTC.1                          ' SPI CS to ADS7846 CS (Active Low)
$endif 

$ifndef Touch_Sensitivity               
    $define Touch_Sensitivity 64                   ' Sensitivity of the touchscreen to detect a stylus landing on it
$endif
'
' Screen coordinates for touch calibration (alter as required)
'    
    $define cScreen_Xminimum 370                    ' Raw ADC value from ADS chip for extreme left of the display
    $define cScreen_Yminimum 350                    ' Raw ADC value from ADS chip for extreme right of the display
    
    $define cScreen_Xmaximum 3830                   ' Raw ADC value from ADS chip for extreme top of the display
    $define cScreen_Ymaximum 3850                   ' Raw ADC value from ADS chip for extreme bottom of the display
    
    Symbol cScreen_Yrange = (cScreen_Ymaximum - cScreen_Yminimum)     ' Calculate Y ADC range across the LCD
    Symbol cScreen_Xrange = (cScreen_Xmaximum - cScreen_Xminimum)     ' Calculate X ADC range across the LCD
    
    Dim ADS_wAccum As Word System
    Dim ADS_bControl As Byte System
'
' Define some variables used by the Touchscreen subroutines
'
    Dim ADS_wXlocation As Word                      ' 16-bit X pos location (0 to 239)
    Dim ADS_bXlocation As ADS_wXlocation.Byte0      ' 8-bit X pos location (0 To 239)
    Dim ADS_wYlocation As Word                      ' 16-bit Y pos location (0 To 319)
    
    Dim ADS_wTemp As Word System 
    Dim ADS_dTemp As PP_dAddress.Dword              ' Create an alias to a variable used in ILI9320.inc
    
    Symbol cAmountOfSamples = 8                     ' Amount of samples to take for the averaging. Maximum 8
        
    Dim ADS_bIndex As Byte System
    Dim ADS_wAdres As ADRESL.Word                   ' The raw ADC result given by the ADS7846 touch screen controller chip
'
' ADS7846 Control Byte values
'    
    Symbol ADS_START          = %10000000           ' Start bit (always set)
    Symbol ADS_A2             = %01000000           ' A2 bit
    Symbol ADS_A1             = %00100000           ' A1 bit
    Symbol ADS_A0             = %00010000           ' A0 bit
    Symbol ADS_MODE_8         = %00001000           ' Mode bit for 8-bit operation
    Symbol ADS_MODE_12        = %00000000           ' Mode bit for 12-bit operation
    Symbol ADS_SINGLE         = %00000100           ' Sample setting bit for single ended operation
    Symbol ADS_DIFF           = %00000000           ' Sample setting bit for differential operation
    Symbol PWRDWN_CONV_PEN_ON = %00000000           ' Power-Down Between Conversions. When each
                                                    ' conversion is finished, the converter enters a
                                                    ' low-power mode. At the start of the next conversion,
                                                    ' the device instantly powers up to full power.
                                                    ' There is no need for additional delays to assure
                                                    ' full operation and the very first conversion is
                                                    ' valid. The Y switch is on when in power-down.
                                                    ' PENIRQ is enabled
    Symbol REF_OFF_PEN_OFF       = %00000001        ' Reference is off and ADC is on. PENIRQ is disabled
    Symbol REF_OFF_ADC_ON_PEN_ON = %00000010        ' Reference is on and ADC is off. PENIRQ is enabled
    Symbol PWR_ON_REF_ON_ADC_ON_PEN_OFF = %00000011 ' Device is always powered. Reference is on and ADC is on. PENIRQ is disabled
    
    Symbol ADS_MSR_Z1 = %00110000
    Symbol ADS_MSR_Z2 = %01000000
    Symbol ADS_MSR_Y =  %00010000
    Symbol ADS_MSR_X =  %01010000
    
    Symbol cMeasureZ1 = ADS_START | ADS_MSR_Z1 | ADS_MODE_12 | ADS_DIFF | PWR_ON_REF_ON_ADC_ON_PEN_OFF   ' Control byte for a measurement of the Z1 axis
    Symbol cMeasureZ2 = ADS_START | ADS_MSR_Z2 | ADS_MODE_12 | ADS_DIFF | PWR_ON_REF_ON_ADC_ON_PEN_OFF   ' Control byte for a measurement of the Z2 axis
    Symbol cMeasureY = ADS_START | ADS_MSR_Y | ADS_MODE_12 | ADS_DIFF | PWR_ON_REF_ON_ADC_ON_PEN_OFF     ' Control byte for a measurement of the X axis
    Symbol cMeasureX = ADS_START | ADS_MSR_X | ADS_MODE_12 | ADS_DIFF | PWR_ON_REF_ON_ADC_ON_PEN_OFF     ' Control byte for a measurement of the Y axis
'
' Helper defines
'   
$ifndef True
    $define True 1
$endif
$ifndef False
    $define False 0
$endif

$define tStylusOff False
$define tStylusOn True

'--------------------------------------------------------------------    
    GoTo ADS_Main                                   ' Jump over the subroutines and initialise the SPI interface                        
'--------------------------------------------------------------------
' Helper routine to take a measurement from the touch sensor and average the results
' Input     : WREG holds the control byte
' Output    : ADS_wAdres holds the ADC value read from the touch chip
' Notes     : Performs a simple averaging mechanism on each ADC result
'
$define hTouch_GetSample(pControl) '
    WREG = pControl '
    _Get_Sample

_Get_Sample Macro- 
    GoSub __Get_Sample
Endm

#ifMacro- _Get_Sample, _Touch_Active, _Touch_Get
__Get_Sample:
    
    ADS_bControl = WREG                             ' Get the control byte into its working variable
    ADS_wAccum = 0
    ADS_bIndex = 0
    Repeat
        Clear ADS_CS                                ' Enable the SPI interface  
        WriteSPI(ADS_bControl)                      ' Send the control Byte
        ADS_wAdres.Byte1 = ReadSPI()                ' Get the high byte 
        ADS_wAdres.Byte0 = ReadSPI()                ' Get the low byte
        Set ADS_CS                                  ' Disable the SPI interface
        ADS_wAdres = ADS_wAdres >> 3
        ADS_wAdres.12 = 0
        ADS_wAccum = ADS_wAccum + ADS_wAdres        ' Add the ADC result to the accumualtor
        DelayUS 500                                 ' Allow the screen to settle before taking the next reading
        Inc ADS_bIndex
    Until ADS_bIndex >= cAmountOfSamples
    ADS_wAdres = ADS_wAccum / cAmountOfSamples
    Return
#endIfMacro-

'------------------------------------------------------------------------------------------------
' Test if the screen has been touched
' Input     : None
' Output    : WREG returns 0 if the display is not being touched by the stylus
'           : ADS_wAdres holds the Z axis of the screen. i.e. touch pressure value
' Notes     : Uses the Z1 axis of the touch screen
'           : Threshold value for stylus detect is user defined in constant Touch_Sensitivity
'
$define Touch_Active() _Touch_Active

_Touch_Active Macro- \Byte
    GoSub __Touch_Active
    Return_Wreg
Endm

#ifMacro- _Touch_Active
__Touch_Active:
    hTouch_GetSample(cMeasureZ1)                    ' Z1 Conversion, 12-bit into ADS_wAdres
    If ADS_wAdres >= Touch_Sensitivity Then         ' Is the Z axis value greater than the set threshold?
        WREG = tStylusOn                            ' Yes. So the screen must have been touched   
        Return                                      ' Exit the routine immediately
    EndIf
    WREG = tStylusOff                               ' Return the defualt setting for the screen not being touched
    Return
#endIfMacro-

'--------------------------------------------------------------------
' Get the Xpos and Ypos positions
' Input     : None
' Output    : ADS_wXlocation holds the pixel X position of the stylus (0 to 239)
'           : ADS_wYlocation holds the pixel Y position of the stylus (0 to 319)
'           : WREG returns holding false (0) if the stylus is outside the LCD's boundaries
'           : Otherwise WREG returns true (1)
' Notes     : ADS_wXlocation can be picked up by using the Touch_X macro
'           : ADS_wYlocation can be picked up by using the Touch_Y Macro
'
$define Touch_Get() _Touch_Get

_Touch_Get Macro- \Byte
    GoSub __Touch_Get
    Return_Wreg
Endm

#ifMacro- _Touch_Get
__Touch_Get:      
    hTouch_GetSample(cMeasureX)                         ' X Conversion, 12-bit into ADS_wAdres
    ADS_dTemp = (ADS_wAdres - cScreen_Xminimum) * 240   ' \ Calculate the stylus X location (n * 240)
    ADS_wXlocation = ADS_dTemp / cScreen_Xrange         ' /
    If ADS_bXlocation > 239 Then                        ' Is the stylus outside the LCD's X boundary?
        WREG = False                                    ' Yes. So indicate if the stylus value is outside the LCD's X boundary
        Return                                          ' Exit the routine immediately
    EndIf   
    hTouch_GetSample(cMeasureY)                         ' Y Conversion, 12-bit into ADS_wAdres 
    ADS_dTemp = (ADS_wAdres - cScreen_Yminimum) * 320   ' \ Calculate the stylus Y location (n * 320)
    ADS_wYlocation = ADS_dTemp / cScreen_Yrange         ' /
    If ADS_wYlocation > 319 Then                        ' Is the stylus outside the LCD's Y boundary?
        WREG = False                                    ' Yes. So indicate If the stylus value is outside the LCD's Y boundary
        Return                                          ' Exit the routine immediately
    EndIf
    WREG = True                                         ' Exit with WREG holding True for a good touch
    Return
#endIfMacro-
'----------------------------------------------------------------------------------------------
$define Touch_X() ADS_wXlocation.Byte0                  ' 8-bit X pos location (0 To 239)
'----------------------------------------------------------------------------------------------
$define Touch_Y() ADS_wYlocation                        ' 16-bit Y pos location (0 To 319)
'----------------------------------------------------------------------------------------------
' Check of the Stylus is within a certain area of the LCD
' Input     : WREG is the Top-Left X coordinate of the area to check
'           : Glcd_wYpos_S is the Top-Left Y coordinate of the area to check
'           : Glcd_bXpos_E is the Bottom-Right X coordinate of the area to check
'           : Glcd_wYpos_E is the Bottom-Right Y coordinate of the area to check
' Output    : WREG = 1 if Stylus is within the area, else 0
' Notes     : Re-uses variables declared in ILI9320.inc
' Syntax    :
' Var = Touch_Hotspot(Top-Left X coord, Top-Left Y coord, Bottom-Right X coord, Bottom-Right Y coord)
'
' pXstart, pYstart
'           X------------------------
'           |                       |
'           |                       |
'           |                       |
'           |                       |
'           ------------------------X
'                           pXend, pYend
'   
$define Touch_HotSpot(pXstart, pYstart, pXend, pYend) _mHotSpot pXstart, pYstart, pXend, pYend 

_mHotSpot Macro- pXposStart, pYposStart, pXposEnd, pYposEnd \Byte
    
    #if((Prm_2 == Num8) || (Prm_2 == Num16) || (Prm_2 == Num32))
        Num_Word pYposStart , Glcd_wYpos_S
    #endif
    #if(Prm_2 == Byte)
        Byte_Word pYposStart , Glcd_wYpos_S
    #endif
    #if((Prm_2 == Word) || (Prm_2 == Dword))
        Word_Word pYposStart , Glcd_wYpos_S
    #endif

    #if((Prm_3 == Num8) || (Prm_3 == Num16) || (Prm_3 == Num32))
        Num_Byte pXposEnd , Glcd_bXpos_E
    #endif
    #if((Prm_3 == Byte) || (Prm_3 == Word) || (Prm_3 == Dword))
        Byte_Byte pXposEnd , Glcd_bXpos_E
    #endif 

    #if((Prm_4 == Num8) || (Prm_4 == Num16) || (Prm_4 == Num32))
        Num_Word pYposEnd , Glcd_wYpos_E
    #endif
    #if(Prm_4 == Byte)
        Byte_Word pYposEnd , Glcd_wYpos_E
    #endif 
    #if((Prm_4 == Word) || (Prm_4 == Dword))
        Word_Word pYposEnd , Glcd_wYpos_E
    #endif 
    
    #if((Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32))
        Num_Wreg pXposStart
    #endif
    #if((Prm_1 == Byte) || (Prm_1 == Word) || (Prm_1 == Dword))
        Byte_Wreg pXposStart
    #endif    
    GoSub _HotSpot   
    Return_Wreg
    Endm

#ifMacro- _mHotSpot
_HotSpot:
    If ADS_bXlocation >= WREG Then
        If ADS_wYlocation >= Glcd_wYpos_S Then
            If ADS_bXlocation <= Glcd_bXpos_E Then
                If ADS_wYlocation <= Glcd_wYpos_E Then
                    WREG = True
                    Return
                EndIf
            EndIf
        EndIf
    EndIf
    WREG = False
    Return
#endIfMacro-
'----------------------------------------------------------------------------------------------
' Check if the Stylus is within a certain area of the LCD with the coordinates held in a code memory table
' Input     : Reads position data from a cdata table pointed to by TBLPTRL\H
' Output    : WREG = 1 if Stylus is within the area, else 0
' Notes     : Table format is Xpos_Start(8-bit), Ypos_Start(16-bit), Xpos_End(8-bit), Ypos_End(16-bit)
'           : Cdata Byte Xpos_Start, Word, Ypos_Start, Byte Xpos_End, Word Ypos_End
' Syntax:
' Var = Touch_HotspotTable(pTableAddress)
'
' Xpos_Start, Ypos_Start
'           X------------------------
'           |                       |
'           |                       |
'           |                       |
'           |                       |
'           ------------------------X
'                           Xpos_End, Ypos_End
'   
$define Touch_TableHotSpot(pTableAddress) _mTableHotSpot pTableAddress 

_mTableHotSpot Macro- pTableAddress \Byte   
    
    #if(Prm_1 == Label)
        Label_Word pTableAddress, ILI_wTblPtr
    #endif
    #if((Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32))
        Num_Word pXposStart
    #endif
    #if(Prm_1 == Byte)
        Byte_Word pTableAddress, ILI_wTblPtr
    #endif
    #if((Prm_1 == Word) || (Prm_1 == Dword))
        Word_Word pTableAddress, ILI_wTblPtr
    #endif  
    GoSub _TableHotSpot
    Return_Wreg
    Endm

#ifMacro- _mTableHotSpot
_TableHotSpot:      
    Tblrd*+                                         ' Read 8-bit X start position
    If ADS_bXlocation >= TABLAT Then                ' Is the stylus X location greater than the required X start position?
        Tblrd*+                                     ' Yes. So read the low byte of Y start position
        Glcd_wParam.Byte0 = TABLAT                  ' Store it
        Tblrd*+                                     ' Read the high byte of Y start position
        Glcd_wParam.Byte1 = TABLAT                  ' Store it
        If ADS_wYlocation >= Glcd_wParam Then       ' Is the stylus Y location greater than the required Y start position?
            Tblrd*+                                 ' Yes. So read the 8-bit X end position
            If ADS_bXlocation <= TABLAT Then        ' Is the stylus X location less than the required X end position?
                Tblrd*+                             ' Yes. So read the low byte of Y end position
                Glcd_wParam.Byte0 = TABLAT          ' Store it
                Tblrd*+                             ' Read the high byte of Y End position
                Glcd_wParam.Byte1 = TABLAT          ' Store it
                If ADS_wYlocation <= Glcd_wParam Then ' Is the stylus Y location less than the required Y end position? 
                    WREG = True                     ' Yes. So return a positive result
                    Return                          ' Exit the routine
                EndIf
            EndIf
        EndIf
    EndIf
    WREG = False
    Return
#endIfMacro-
'--------------------------------------------------------------------
' Initialise the SPI interface to the ADS7846 Touch Screen Controller
' Input     : None
' Output    : None
' Notes     : Opens the SPI peripheral in mode 0:0 at 4MHz
'
$define Touch_Init() _Touch_Init

_Touch_Init Macro-
    GoSub __Touch_Init
    Endm
 
#ifMacro- _Touch_Init   
__Touch_Init:
'
' Setup the ADS7846 SPI lines
'
    High ADS_CS                                     ' Hold the ADS chip's SPI interface off   
    OpenSPI(SPI_FOSC_16, MODE_00, SMPMID)           ' Open the SPI peripheral (4MHz, Mode 0:0, Sample in middle)   
    Return
#endIfMacro-    
'--------------------------------------------------------------------
ADS_Main:  
    Touch_Init()
$endif
    
